# -*- coding: utf-8 -*-
#
# Copyright (c) 2012 by nils_2 <weechatter@arcor.de>
#
# customize your title/status/input bar for each buffer
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# This script deletes weechatlog-files by age or size
# YOU ARE USING THIS SCRIPT AT YOUR OWN RISK!
#
# 2012-01-20: nils_2, (freenode.#weechat)
#       0.1 : initial release
#
# Development is currently hosted at
# https://github.com/weechatter/weechat-scripts

try:
    import weechat,re

except Exception:
    print "This script must be run under WeeChat."
    print "Get WeeChat now at: http://www.weechat.org/"
    quit()

SCRIPT_NAME     = "customize_bar"
SCRIPT_AUTHOR   = "nils_2 <weechatter@arcor.de>"
SCRIPT_VERSION  = "0.1"
SCRIPT_LICENSE  = "GPL"
SCRIPT_DESC     = "customize your title/status/input bar for each buffer"

OPTIONS         =  {'default.title'     : 'weechat.bar.title.items',
                    'default.status'    : 'weechat.bar.status.items',
                    'default.input'     : 'weechat.bar.input.items',}
DEFAULT_OPTION  =  {}                                                                   # save default options from weechat.bar.*.items
# ================================[ programm ]===============================
def buffer_switch(data, signal, signal_data):
    full_name = weechat.buffer_get_string(signal_data,'full_name')                      # get full_name of current buffer
    if full_name == '':                                                                 # upps, something totally wrong!
        return weechat.WEECHAT_RC_OK

    for option in OPTIONS.keys():
        option = option.split('.')
        customize_plugin = weechat.config_get_plugin('%s.%s' % (option[1], full_name))  # for example: title.irc.freenode.#weechat
        if customize_plugin:                                                            # option exists
            config_pnt = weechat.config_get('weechat.bar.%s.items' % option[1])
#            current_bar = weechat.config_string(weechat.config_get('weechat.bar.%s.items' % option[1]))
            weechat.config_option_set(config_pnt,customize_plugin,1)                    # set customize_bar
        else:
            current_bar = weechat.config_string(weechat.config_get('weechat.bar.%s.items' % option[1]))
            default_plugin = weechat.config_get_plugin('default.%s' % option[1])        # option we are looking for
            if default_plugin == '':                                                    # default_plugin removed by user?
                weechat.config_set_plugin('default.%s' % option[1],DEFAULT_OPTION[option[1]]) # set default_plugin again!
            if current_bar != default_plugin:
                config_pnt = weechat.config_get('weechat.bar.%s.items' % option[1])
                weechat.config_option_set(config_pnt,default_plugin,1)                  # set customize_bar

    return weechat.WEECHAT_RC_OK

def customize_cmd_cb(data, buffer, args):
    args = args.lower()
    argv = args.split(None)
    if (args == '') or (len(argv) != 2):
        return weechat.WEECHAT_RC_OK
    if argv[1] not in 'title status input':
        return weechat.WEECHAT_RC_OK

    full_name = weechat.buffer_get_string(buffer,'full_name')                           # get full_name of current buffer

    if argv[0] == 'add':
        if weechat.config_get_plugin('%s.%s' % (argv[1],full_name)):
            return weechat.WEECHAT_RC_OK
        else:
            default_plugin = weechat.config_get_plugin('default.%s' % (argv[1]))
            bar = weechat.config_set_plugin('%s.%s' % (argv[1],full_name),default_plugin)  # set default bar
    elif argv[0] == 'del':
        weechat.config_unset_plugin('%s.%s' % (argv[1],full_name))
    return weechat.WEECHAT_RC_OK

def customize_bar_completion_cb(data, completion_item, buffer, completion):
    for option in OPTIONS.keys():
        option = option.split('.')
        weechat.hook_completion_list_add(completion, option[1], 0, weechat.WEECHAT_LIST_POS_SORT)
    return weechat.WEECHAT_RC_OK

def shutdown_cb():
    # write back default options to original options, then quit...
    for option in OPTIONS.keys():
        option = option.split('.')
        default_plugin = weechat.config_get_plugin('default.%s' % option[1])
        config_pnt = weechat.config_get('weechat.bar.%s.items' % option[1])
        weechat.config_option_set(config_pnt,default_plugin,1)
    return weechat.WEECHAT_RC_OK
# ================================[ config ]===============================
def init_options():
    # check out if a default item bar exists
    for option,value in OPTIONS.items():
        if not weechat.config_get_plugin(option):
            default_bar = weechat.config_string(weechat.config_get(value))# get original option
            weechat.config_set_plugin(option, default_bar)
            default_option = option.split('.')
            default_bar_value = weechat.config_string(weechat.config_get('weechat.bar.%s.items' % default_option[1]))
            DEFAULT_OPTION[default_option[1]] = default_bar_value
        else:
            default_option = option.split('.')
            default_bar_value = weechat.config_string(weechat.config_get('weechat.bar.%s.items' % default_option[1]))
            DEFAULT_OPTION[default_option[1]] = default_bar_value
# ================================[ main ]===============================
if __name__ == "__main__":
    if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, 'shutdown_cb', ''):
        version = weechat.info_get('version_number', '') or 0
        weechat.hook_command(SCRIPT_NAME,SCRIPT_DESC,
        'add <title|status|input> || del <title|status|input>',
        'add <title|status|input>: add a (default) customize bar for current buffer\n'
        'del <title|status|input>: delete customize bar for current buffer\n\n'
        'Options:\n'
        '  plugins.var.python.customize_bar.default.title : stores the default items from weechat title bar.\n'
        '  plugins.var.python.customize_bar.default.status: stores the default items from weechat status bar.\n'
        '  plugins.var.python.customize_bar.default.input : stores the default items from weechat input bar.\n'
        '  plugins.var.python.customize_bar.(title|status|input).<full_buffer_name> : stores the customize bar items for this buffer\n\n'
        'CAVE: Do not delete options \"plugins.var.python.customize_bar.default.*\" as long as script is running...\n',
        'add %(plugin_customize_bar) %-|| del %(plugin_customize_bar) %-',
        'customize_cmd_cb', '')
        init_options()
        weechat.hook_signal('buffer_switch','buffer_switch','')
        weechat.hook_completion('plugin_customize_bar', 'customize_bar_completion', 'customize_bar_completion_cb', '')
        weechat.command('','/window refresh')


# -*- coding: utf-8 -*-
# Copyright (C) 2011  delwin <delwin@skyehaven.net>
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#based on the 'lnotify' script for weechat, which was based on 'notify'
#
#the main reason to use this script would be if you use tmux or screen
#with weechat and would like to receive desktop notifications on multiple
#computers (regardless of whether or not you are connected to weechat on
#any of those systems.) however, there is a reason to use this if you
#only use one system for weechat. if you restart X and then reconnect
#to weechat, notifications will no longer work because of dbus errors.
#this solution solves that problem at the expense of some extra setup.
#
#IMPORTANT:
#use of this script assumes that you have:
#1) /usr/bin/notify-send
#2) ssh configured to use key identification (including localhost!)
#
#ALSO IMPORTANT:
#see comments below in the default settings section for description of options
#and tips. pay particular attention to the addresses section if you need to
#specify non-default ports or non-default display settings
#
#NOT so important:
#if you define icons to show up in the notifications, this script will expect
#to find them in the same place on all systems. missing icons will not stop the
#notification from displaying, however. (this also applies for any extra
#commands you add to be executed after notify-send. sound files and utilities
#have to exist on the machine receiving the command in order to work)
#
#
#changelog:
#v0.2.3 - Sébastien Helleu <flashcode@flashtux.org>
#         change hook_print callback argument type of displayed/highlight
#         (WeeChat >= 1.0)
#v0.2.2 - <ldvx@freenode> fixed bug in (1)  which didn't allow user to get
#         notifications on private messages if irc.look.nick_prefix or
#         irc.look.nick_suffix were used. (2012-04-27)
#v0.2.1 - added help messages and hint for empty addresses option (2011-10-15)
#v0.2.0 - added several options, including: proper weechat options
#         for multiple addresses, ignore strings, urgencies, images,
#         and extra commands to be sent along with notifications.
#         shifted handling of DISPLAY to the address portion of the command.
#         submitted to weechat scripts. (2011-10-14)
#v0.1.0 - converted lnotify to work via ssh. accepts multiple addresses
#         hardcoded into the script. (2011-09-18)
#
#TODO... if i get bored enough/anyone actually requests these things:
#         add notification whitelist, only send notifications matching whitelist
#         set up proper /sshnotify <args> command to manually send notifications
#         set up new definitions for messages so that extra_commands can use them?
#         toggle to suppress notifications when away
#         toggle to show server and channel names in notifications
#         toggle to show notifications for dcc?
#
import weechat, string, subprocess, re

weechat.register("sshnotify", "delwin", "0.2.3", "GPL3", "the overkill desktop notification solution", "", "")

#options which can be defined with /set plugins.var.python.sshnotify.foo
settings = {
    "show_highlight"     : "on", #trigger on highlighted nick/word
    "show_priv_msg"      : "on", #trigger on private message
    "ignore"             : "", #comma seperated list of strings to ignore,
                               #e.g. if using bitlbee, you might want to add
                               #'@root: jabber,@root: otr' to avoid spam on startup
    "pm-urgency"         : "critical", #notification urgency for private messages. valid values are: low,normal,critical
    "pm-image"           : "", #location of notification icon for private messages, optional
    "mention-urgency"    : "normal", #notification urgency for highlighted strings, valid values are: low,normal,critical
    "mention-image"      : "", #location of notification icon for highlighted strings, optional
    "extra_commands"     : "", #extra commands to pass via ssh, appeneded after notify command.
                               #note: you need to include && to execute the commands sequentially
                               #for example, in order to have espeak announce that you have a new message and then play an audio file:
                               #/set plugins.var.python.sshnotify.extra_commands && espeak 'hey, you have a new message' && aplay whatever.wav'
    "addresses"          : "localhost", #comma delimited lists of addresses (and options) to use via ssh
                                        #default is localhost. if no address is specified, this script won't do anything for you
                                        #if you set addresses to "", it will reset to localhost the next time it loads.
                                        #if you need to specify a different display, add DISPLAY=:1 (or the appropriate number)
                                        #by default, sshnotify will use DISPLAY=:0, so there is no need to specify that in your address string
                                        #other ssh options can be added to each address:
                                        #/set plugins.var.python.sshnotify.addresses localhost,foo@bar,-p 1234 bar@foo DISPLAY=:1
                                        #will send a notification to 3 places, you@localhost, foot@bar, and bar@foo on port 1234 on display 1

}

# Init everything
for option, default_value in settings.items():
    if weechat.config_get_plugin(option) == "":
        weechat.config_set_plugin(option, default_value)

# Hook privmsg/hilights
weechat.hook_print("", "irc_privmsg", "", 1, "get_notified", "")

# Functions

def get_addresses():
    addies = weechat.config_get_plugin('addresses')
    if addies == '':
        weechat.prnt("","You need to specify a destination if you want notifications sent. ")
        weechat.prnt("","hint: /set plugins.var.python.sshnotify.addresses localhost")
        return []
    else:
        return addies.split(',')

#if there are any strings
def get_ignore():
    ignores = weechat.config_get_plugin('ignore')
    if ignores == '':
        return []
    else:
        return ignores.split(',')

#notification routine
def get_notified(data, bufferp, uber_empty, tagsn, isdisplayed,
        ishilight, prefix, message):

    #if message contains and ignored string, don't send the notification
    ilist = get_ignore()
    for i in ilist:
        if re.search(i,prefix + ": " + message):
            return weechat.WEECHAT_RC_OK

    extracommands = weechat.config_get_plugin('extra_commands')
    #set a default value for DISPLAY. this should be fine for almost everyone
    dispnum = "DISPLAY=:0"

    #if the message came in via private message...
    if (weechat.buffer_get_string(bufferp, "localvar_type") == "private" and
            weechat.config_get_plugin('show_priv_msg') == "on"):
        buffer = (weechat.buffer_get_string(bufferp, "short_name") or
                weechat.buffer_get_string(bufferp, "name"))

        #set notification image
        if weechat.config_get_plugin('pm-image') != "":
            imagestring = "--icon=" + str(weechat.config_get_plugin('pm-image')) + " "
        else:
            imagestring = ""

        #set notification urgency
        if weechat.config_get_plugin('pm-urgency') == "low":
            urgencystring = "--urgency=low "
        elif weechat.config_get_plugin('pm-urgency') == "critical":
            urgencystring = "--urgency=critical "
        else:
            urgencystring = "--urgency=normal "

        uistring = urgencystring + imagestring

        # (1) if buffer == prefix was used here, pressumibly to avoid notifications when on own
        # messages, checking for the tag notify_private has the same effect, and the user can
        # set irc.look.nick_suffix or irc.look.nick_prefix this way.
        if "notify_private" in tagsn.split(","):
            #the ' character currently needs changed to something else or the message formatting fails
            #substituting " for '
            #notification title
            prefix = re.sub("'",'"',prefix)
            #escaping all special characters so that the message formatting doesn't fail
            prefix = re.escape(prefix)
            #notification message
            message = re.sub("'",'"',message)
            message = re.escape(message)
            #setting the command which will be passed by ssh to push the notification
            #note the DISPLAY variable is now accounted for in the ssh part of the command rather than this location
            disp = '"/usr/bin/notify-send ' + uistring + '\'In PM\' \'' + prefix + ': ' + message + '\' ' + extracommands + '\"'

          #fire when ready
            alist = get_addresses()
            for a in alist:
                if a != '':
                    #first check to see if DISPLAY is set in an address listing
                    if re.search('DISPLAY\=',a):
                        #if yes, do not set the default value defined above
                        dispnum = ""
                    #generate the ssh portion of the command to send
                    com = "ssh -X " + a + " " + dispnum + " "
                    #add all the bits together and send the notification. time out if it can't connect in 5 seconds
                    weechat.hook_process(com + disp, 5000, "", "")
                    #just a debug message
                    #print(com + disp)


    #if the message comes from a highlight rather than private message
    elif (int(ishilight) and
            weechat.config_get_plugin('show_highlight') == "on"):
        buffer = (weechat.buffer_get_string(bufferp, "short_name") or
                weechat.buffer_get_string(bufferp, "name"))
        #convert ' to " and escape special characters so the ssh command doesn't puke
        buffer = re.sub("'",'"',buffer)
        buffer = re.escape(buffer)
        #notification title
        prefix = re.sub("'",'"',prefix)
        prefix = re.escape(prefix)
        #notification message
        message = re.sub("'",'"',message)
        message = re.escape(message)

        #set notification image
        if weechat.config_get_plugin('mention-image') != "":
            imagestring = "--icon=" + weechat.config_get_plugin('mention-image') + " "
        else:
            imagestring = ""

        #set notification urgency
        if weechat.config_get_plugin('mention-urgency') == "low":
            urgencystring = "--urgency=low "
        elif weechat.config_get_plugin('mention-urgency') == "critical":
            urgencystring = "--urgency=critical "
        else:
            urgencystring = "--urgency=normal "

        uistring = urgencystring + imagestring
        #adding the notify-send command bits together
        disp = '"/usr/bin/notify-send ' + uistring + '\'In ' + buffer + '\' \'' + prefix + ': ' + message + '\'' + extracommands + '\"'

        #decide where to send the notification
        alist = get_addresses()
        for a in alist:
            if a != '':
                #if display is set in the address, do not apply default value defined above
                if re.search('DISPLAY\=',a):
                    dispnum = ""
                #adding the ssh command bits together
                com = "ssh -X " + a + " " + dispnum + " "
                #add all the bits together and send the notification. timeout in 5 seconds
                weechat.hook_process(com + disp, 5000, "", "")
                #just a debug message
                #print(com + disp)


    return weechat.WEECHAT_RC_OK

#right now this is just to enable /help sshnotify
#might be a better way to do this but i might extend it
#to allow /sshnotify to send notications directly with
#arguments as the messages
def notifying(data,buffer,args):
    weechat.prnt("","the command /sshnotify won't do much for you...yet. see /help sshnotify for info on how to use this plugin")
    return weechat.WEECHAT_RC_OK

#the help message from /help sshnotify
hook = weechat.hook_command(
    "sshnotify","overkill desktop notification","",
"""
  This script allows you to send desktop
  notifications of private messages and
  highlighted strings to one or more
  computers via ssh. This script expects
  ssh key identification to be set up and
  /usr/bin/notify-send on the systems to
  which you are sending notifications.

  It is quite configurable, allowing you
  to specify any ssh options you might find
  necessary (e.g. custom ports and
  non-default DISPLAY settings.) It allows
  you to specify strings to be ignored to
  help cut down on notification spam. Both
  of these settings accept comma delimited lists.

  Examples:
  for multiple ssh connections:
  /set plugins.var.python.sshnotify.addresses
      localhost,foo@bar,-p 1234 bar@foo DISPLAY=:1
  (sends notifications to you@localhost, foo@bar,
   and bar@foo on port 1234 at DISPLAY 1)

  for multiple ignore values:
  /set plugins.var.python.sshnotify.ignore
      @root: jabber,@root: otr,ignore this string
  (I use these values to eliminate notification
   spam from bitlbee when I connect)

  It also allows you to chain extra commands
  to be executed after the notify-send command,
  like so:
  /set plugins.var.python.sshnotify.extra_commands
      && espeak 'you have a new message && aplay somerandom.wav
  (note: this is not comma delimited and expects '&&'
   in front of any command you wish to execute,
   including the first one.)

  You can also set the urgency level and icons
  used for private messages or highlighted
  strings (the 'mention' configure options.)
  Valid urgency levels are 'low','normal', and
  'critical'. Icons need to be on the system
  receiving the notification in order to display,
  but if they are missing it does not interfere
  with the functionality of this script.

  I should also note that apostrophes in messages
  will be converted to double quotes in your
  notifications due to formatting issues with
  the ssh/notify-send command.
    """
    ,"",'notifying','')



# -*- coding: utf-8 -*-
###
# Copyright (c) 2010 by Elián Hanisch <lambdae2@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###

###
# Python interpreter for WeeChat, and module for debugging python scripts.
#
#   Commands (see detailed help with /help in WeeChat):
#   * /pybuffer: Opens a python interpreter buffer.
#
# For debug a script, insert these lines after script's register() call.
#
#   import pybuffer
#   debug = pybuffer.debugBuffer(globals(), "buffer_name")
#
# Then, after loading your script, try "dir()" in the new "buffer_name" buffer, it should
# display the script's global functions and variables.
# This module should be in your python scripts path.
#
# Session example:
#
#   >>> buffer_search('irc', 'freenode.#weechat')
#   '0x9ca4ce0'
#   >>> b = buffer_search('irc', 'freenode.#weechat')
#   >>> b
#   '0x9ca4ce0'
#   >>> buffer_get(b, 'input')
#   Traceback (most recent call last):
#     File "<console>", line 1, in <module>    
#   NameError: name 'buffer_get' is not defined
#   >>> search('buffer')
#   ['buffer_clear', 'buffer_close', 'buffer_get_integer', 'buffer_get_pointer', 'buffer_get_string',
#   'buffer_merge', 'buffer_new', 'buffer_search', 'buffer_search_main', 'buffer_set',
#   'buffer_string_replace_local_var', 'buffer_unmerge', 'current_buffer', 'string_input_for_buffer']
#   >>> buffer_get_string(b, 'input')
#   ''
#   >>> buffer_get_string(b, 'input')
#   'asdasdas hello!'
#   >>> buffer_get_string(b, 'title')
#   'WeeChat, stable: 0.3.3, web: http://www.weechat.org/ | English support channel | Please read
#   doc/faq/quickstart before asking here | Old versions (0.2.6.x and earlier) are not supported any
#   more'
#
#
#   History:
#   2010-11-05
#   version 0.2:
#   * More interperter console.
#   * renamed to pybuffer.
#   * added /pybuffer command (buffer isn't created on script load).
#
#   2010-10-30
#   version 0.1: Initial release
###

SCRIPT_NAME    = "pybuffer"
SCRIPT_AUTHOR  = "Elián Hanisch <lambdae2@gmail.com>"
SCRIPT_VERSION = "0.2"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC    = "Python interpreter for WeeChat and module for debug scripts."

try:
    import weechat
    from weechat import WEECHAT_RC_OK, prnt
    import_ok = True
except ImportError:
    print "This script must be run under WeeChat."
    print "Get WeeChat now at: http://www.weechat.org/"
    import_ok = False

import code, sys, traceback
from fnmatch import fnmatch

def callback(method):
    """This function will take a bound method or function and make it a callback."""
    # try to create a descriptive and unique name.
    func = method.func_name
    try:
        im_self = method.im_self
        try:
            inst = im_self.__name__
        except AttributeError:
            try:
                inst = im_self.name
            except AttributeError:
                raise Exception("Instance %s has no __name__ attribute" %im_self)
        cls = type(im_self).__name__
        name = '_'.join((cls, inst, func))
    except AttributeError:
        # not a bound method
        name = func

    # set our callback
    import __main__
    setattr(__main__, name, method)
    return name


class NoArguments(Exception):
    pass


class ArgumentError(Exception):
    pass


class Command(object):
    """Class for hook WeeChat commands."""
    description, usage, help = "WeeChat command.", "[define usage template]", "detailed help here"
    command = ''
    completion = ''

    def __init__(self):
        assert self.command, "No command defined"
        self.__name__ = self.command
        self._pointer = ''   
        self._callback = ''   

    def __call__(self, *args):
        return self.callback(*args)

    def callback(self, data, buffer, args):
        """Called by WeeChat when /command is used."""
        self.data, self.buffer, self.args = data, buffer, args
        try:
            self.parser(args)  # argument parsing
        except ArgumentError, e:
            error('Argument error, %s' %e)
        except NoArguments:
            pass
        else:
            self.execute()
        return WEECHAT_RC_OK

    def parser(self, args):
        """Argument parsing, override if needed."""
        pass

    def execute(self):
        """This method is called when the command is run, override this."""
        pass

    def hook(self):
        assert not self._pointer, \
                "There's already a hook pointer, unhook first (%s)" %self.command
        self._callback = callback(self.callback)
        pointer = weechat.hook_command(self.command,
                                       self.description,
                                       self.usage,
                                       self.help,
                                       self.completion,
                                       self._callback, '')
        if pointer == '':
            raise Exception, "hook_command failed: %s %s" %(SCRIPT_NAME, self.command)
        self._pointer = pointer

    def unhook(self):
        if self._pointer:
            weechat.unhook(self._pointer)
            self._pointer = ''
            self._callback = ''


class SimpleBuffer(object):
    """WeeChat buffer. Only for displaying lines."""
    _title = ''
    def __init__(self, name):
        assert name, "Buffer needs a name."
        self.__name__ = name
        self._pointer = ''

    def _getBuffer(self):
        # we need to always search the buffer, since there's no close callback we can't know if the
        # buffer was closed.
        buffer = weechat.buffer_search('python', self.__name__)
        if not buffer:
            buffer = self.create()
        return buffer

    def _create(self):
        return weechat.buffer_new(self.__name__, '', '', '', '')

    def create(self):
        buffer = self._create()
        if self._title:
            weechat.buffer_set(buffer, 'title', self._title)
        self._pointer = buffer
        return buffer

    def title(self, s):
        self._title = s
        weechat.buffer_set(self._getBuffer(), 'title', s)

    def clear(self):
        weechat.buffer_clear(self._getBuffer())

    def __call__(self, s, *args, **kwargs):
        self.prnt(s, *args, **kwargs)

    def display(self):
        weechat.buffer_set(self._getBuffer(), 'display', '1')

    def error(self, s, *args):
        self.prnt(s, prefix=weechat.prefix('error'))

    def prnt(self, s, *args, **kwargs):
        """Prints messages in buffer."""
        buffer = self._getBuffer()
        if not isinstance(s, basestring):
            s = str(s)
        if args:
            s = s %args
        try:
            s = kwargs['prefix'] + s
        except KeyError:
            pass
        prnt(buffer, s)

    def prnt_lines(self, s, *args, **kwargs):
        for line in s.splitlines():
            self.prnt(line, *args, **kwargs)


class Buffer(SimpleBuffer):
    """WeeChat buffer. With input and close methods."""
    def _create(self):
        return weechat.buffer_new(self.__name__, callback(self.input), '', callback(self.close), '')

    def _getBuffer(self):
        if self._pointer:
            return self._pointer
        return SimpleBuffer._getBuffer(self)

    def input(self, data, buffer, input):
        return WEECHAT_RC_OK

    def close(self, data, buffer):
        self._pointer = ''
        return WEECHAT_RC_OK


class StreamObject(object):
    def __init__(self, buffer):
        self._content = ''
        self._buffer = buffer

    def write(self, s):
        self._content += s

    def prnt(self, *args, **kwargs):
        if self._content:
            self._buffer.prnt_lines(self._content, *args, **kwargs)
            self._content = ''


class PythonBuffer(Buffer):
    _title = "Python Buffer: use search([pattern]) for a list of objects."
    def __init__(self, name, locals=None):
        Buffer.__init__(self, name)
        self.output = StreamObject(self)
        self.error = StreamObject(self)
        # redirect stdout and stderr
        sys.stdout = self.output
        sys.stderr = self.error
        self.console = code.InteractiveConsole(locals)
        locals = self.console.locals
        # add our 'buildin' functions
        if 'search' not in locals:
            def search(s=''):
                """List functions/objects that match 's' in their name."""
                if '*' not in s:
                    s = '*%s*' %s
                return [ name for name in locals if fnmatch(name, s) ]
            locals['search'] = search

    def _create(self):
        buffer = Buffer._create(self)
        weechat.buffer_set(buffer, 'nicklist', '0')
        weechat.buffer_set(buffer, 'time_for_each_line', '0')
        weechat.buffer_set(buffer, 'localvar_set_no_log', '1')
        self.color_input = weechat.color('green')
        self.color_exc = weechat.color('red')
        self.color_call = weechat.color('cyan')
        weechat.hook_command_run('/input return', callback(self.input_return), buffer)
        # print python and WeeChat version
        prnt(buffer, "Python %s" % sys.version.split(None, 1)[0])
        prnt(buffer, "WeeChat %s" % weechat.info_get('version', ''))
        return buffer

    def __call__(self, s, *args, **kwargs):
        kwargs['prefix'] = self.color_call
        self.prnt(s, *args, **kwargs)
    
    def input_return(self, data, buffer, command):
        # we need to send returns even when there's no input.
        if data == buffer and not weechat.buffer_get_string(buffer, 'input'):
            self.input(data, buffer, '\n')
        return WEECHAT_RC_OK

    def input(self, data, buffer, input):
        """Python code evaluation."""
        try:
            need_more = self.console.push(input)
            if need_more:
                prompt = '%s... ' % self.color_input
            else:
                prompt = '%s>>> ' % self.color_input
            self.prnt(input, prefix=prompt)
            self.output.prnt()
            self.error.prnt(prefix=self.color_exc)
        except:
            trace = traceback.format_exc()
            self.prnt_lines(trace, prefix=self.color_exc)
        return WEECHAT_RC_OK

def debugBuffer(globals, name='debugBuffer'):
    buffer = PythonBuffer(name, globals)
    buffer.create()
    return buffer


class PyBufferCommand(Command):
    command = SCRIPT_NAME
    description = usage = help = ''
    def execute(self):
        buffer = PythonBuffer(SCRIPT_NAME)
        buffer.title("Use \"search([pattern])\" for search WeeChat API functions.")
        # import weechat and its functions.
        buffer.input('', '', 'import weechat')
        buffer.input('', '', 'from weechat import *')
        buffer.display()


if __name__ == '__main__' and import_ok and \
        weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE,
        SCRIPT_DESC, '', ''):

        # we're being loaded as a script.
        help = \
        "Opens a session with a python interpreter-like buffer."\
        " Use function 'search([pattern])' for list objects in current session.\n"\
        "\n"\
        "For debug a script add these lines after the register call:\n"\
        "\n"\
        "  import pybuffer\n"\
        "  debug = pybuffer.debugBuffer(globals(), 'name')\n"\
        "  debug('debug message example')\n"\
        "\n"\
        "You'll be able to execute python code while your script runs.\n"\
        "\n%(b)sWARNING: %(r)s" \
        "This script isn't fool-proof, you're very capable of crashing/freezing "\
        "WeeChat if you aren't careful with the code you run, use at your own risk."\
        " Running python interactive functions such as 'help()' or 'license()' %(b)swill%(r)s hang"\
        " WeeChat." %dict(b=weechat.color('bold'), r=weechat.color('reset'))

        PyBufferCommand.help = help
        PyBufferCommand().hook()


# vim:set shiftwidth=4 tabstop=4 softtabstop=4 expandtab textwidth=100:



# -*- coding: utf-8 -*-
#
# Copyright (C) 2009-2013 Sebastien Helleu <flashcode@flashtux.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

#
# Translate string using Google translate API.
# (this script requires WeeChat >= 0.3.7)
#
# History:
#
# 2013-11-27, luz <ne.tetewi@gmail.com>:
#     version 0.7: switch to json version of google translate api, support
#                  multiple sentences
# 2012-12-11, Sebastien Helleu <flashcode@flashtux.org>:
#     version 0.6: automatically replace old URL (not working any more) by new one
# 2012-10-13, Sebastien Helleu <flashcode@flashtux.org>:
#     version 0.5: fix call to translate API, use hook_process_hashtable
#                  (the script now requires WeeChat >= 0.3.7)
# 2012-01-03, Sebastien Helleu <flashcode@flashtux.org>:
#     version 0.4: make script compatible with Python 3.x
# 2011-08-20, Sebastien Helleu <flashcode@flashtux.org>:
#     version 0.3: fix typo in /help translate
# 2011-03-11, Sebastien Helleu <flashcode@flashtux.org>:
#     version 0.2: get python 2.x binary for hook_process (fix problem when
#                  python 3.x is default python version)
# 2009-10-15, Sebastien Helleu <flashcode@flashtux.org>:
#     version 0.1: initial release
#

SCRIPT_NAME    = 'translate'
SCRIPT_AUTHOR  = 'Sebastien Helleu <flashcode@flashtux.org>'
SCRIPT_VERSION = '0.7'
SCRIPT_LICENSE = 'GPL3'
SCRIPT_DESC    = 'Translate string using Google translate API'

import_ok = True

try:
    import weechat
except:
    print('This script must be run under WeeChat.')
    print('Get WeeChat now at: http://www.weechat.org/')
    import_ok = False

try:
    import sys
    import json
    if sys.version_info >= (3,):
        import urllib.parse as urllib
    else:
        import urllib
except ImportError as message:
    print('Missing package(s) for %s: %s' % (SCRIPT_NAME, message))
    import_ok = False

# script options
translate_settings = {
    'url'     : 'http://translate.google.com/translate_a/t', # google API url
    'from_to' : 'fr_en', # default from_to languages
    'marker'  : '&&',    # translate from this marker in string
}

translate = {
    'hook_process': '',
    'stdout': '',
    'input_before': ['', 0],
    'input_after': ['', 0],
    'options': []
}

def translate_process_cb(data, command, rc, stdout, stderr):
    """Callback reading HTML data from website."""
    global translate
    if stdout != '':
        translate['stdout'] += stdout
    if int(rc) >= 0:
        translated = ''.join([x['trans'] for x in json.loads(translate['stdout'])['sentences']])
        if sys.version_info < (3,):
            translated = translated.encode('utf-8')
        translate['input_before'][0] = weechat.buffer_get_string(weechat.current_buffer(), 'input')
        translate['input_before'][1] = weechat.buffer_get_integer(weechat.current_buffer(), 'input_pos')
        if translate['options']['word']:
            # translate last word of input
            str_input = translate['input_before'][0]
            if str_input:
                pos = str_input.rfind(' ')
                if pos < 0:
                    str_input = translated
                else:
                    str_input = '%s %s' % (str_input[0:pos], translated)
            else:
                str_input = translated
            translate['input_after'][0] = str_input
        else:
            if translate['options']['before_marker']:
                translated = '%s%s' % (translate['options']['before_marker'], translated)
            translate['input_after'][0] = translated
        # set input with translation
        translate['input_after'][1] = len(translate['input_after'][0])
        weechat.buffer_set(weechat.current_buffer(), 'input', translate['input_after'][0])
        weechat.buffer_set(weechat.current_buffer(), 'input_pos', '%d' % translate['input_after'][1])
        translate['hook_process'] = ''
    elif int(rc) == WEECHAT_HOOK_PROCESS_ERROR:
        translate['hook_process'] = ''
    return weechat.WEECHAT_RC_OK

def translate_extract_options(options):
    """
    Extract options from a string, for example (with default from_to == 'fr_en'):
      'le ciel bleu'       => { 'lang': 'fr_en', 'text': 'le ciel bleu' }
      '! the blue sky'     => { 'lang': 'en_fr', 'text': 'the blue sky' }
      'en_it the blue sky' => { 'lang': 'en_it', 'text': 'the blue sky' }
    """
    words = options['string'].split(' ')
    if words:
        if words[0] == '!':
            options['lang'].reverse()
            pos = options['string'].find(' ')
            if pos >= 0:
                options['string'] = options['string'][pos+1:].strip()
            else:
                options['string'] = ''
        else:
            pos = words[0].find('_')
            if pos >= 0 and 5 <= len(words[0]) <= 11:
                options['lang'] = [words[0][0:pos], words[0][pos+1:]]
                pos = options['string'].find(' ')
                if pos >= 0:
                    options['string'] = options['string'][pos+1:].strip()
                else:
                    options['string'] = ''
    words = options['string'].split(' ')
    if words:
        if words[0] == '+':
            options['word'] = True
            pos = options['string'].find(' ')
            if pos >= 0:
                options['string'] = options['string'][pos+1:].strip()
            else:
                options['string'] = ''

def translate_cmd_cb(data, buffer, args):
    """Command /translate."""
    global translate

    # create keys (do NOT overwrite existing keys)
    if args == '-keys':
        rc = weechat.key_bind('default', { 'meta-tmeta-t': '/translate',
                                           'meta-tmeta-r': '/translate !',
                                           'meta-tmeta-w': '/translate +',
                                           'meta-tmeta-u': '/translate <' })
        weechat.prnt('', 'translate: %d keys added' % rc)
        return weechat.WEECHAT_RC_OK

    # undo last translation
    if args == '<':
        current_input = weechat.buffer_get_string(buffer, 'input')
        if translate['input_before'][0] and current_input != translate['input_before'][0]:
            weechat.buffer_set(buffer, 'input', translate['input_before'][0])
            weechat.buffer_set(buffer, 'input_pos', '%d' % translate['input_before'][1])
        elif translate['input_after'][0] != '' and current_input != translate['input_after'][0]:
            weechat.buffer_set(buffer, 'input', translate['input_after'][0])
            weechat.buffer_set(buffer, 'input_pos', '%d' % translate['input_after'][1])
        return weechat.WEECHAT_RC_OK

    # default options
    translate['options'] = { 'lang': weechat.config_get_plugin('from_to').split('_'),
                             'word': False,
                             'before_marker': '',
                             'string': args }

    # read options in command arguments
    translate_extract_options(translate['options'])

    # if there's no string given as argument of command, then use buffer input
    extract = False
    if translate['options']['string'] == '':
        translate['options']['string'] = weechat.buffer_get_string(buffer, 'input')
        extract = True

    # keep only text after marker for translation
    marker = weechat.config_get_plugin('marker')
    if marker != '':
        pos = translate['options']['string'].find(marker)
        if pos >= 0:
            translate['options']['before_marker'] = translate['options']['string'][0:pos]
            translate['options']['string'] = translate['options']['string'][pos+len(marker):]
            extract = True

    # read options in text
    if extract:
        translate_extract_options(translate['options'])

    # keep only last word if option "1 word" is enabled
    if translate['options']['word']:
        translate['options']['string'] = translate['options']['string'].split(' ')[-1]

    # no text to translate? exit!
    if not translate['options']['string']:
        return weechat.WEECHAT_RC_OK

    # cancel current process if there is one
    if translate['hook_process']:
        weechat.unhook(translate['hook_process'])
        translate['hook_process'] = ''

    # translate!
    url = '%s?%s' % (weechat.config_get_plugin('url'), urllib.urlencode({'client': 'a',
                                                                         'sl': translate['options']['lang'][0],
                                                                         'tl': translate['options']['lang'][1],
                                                                         'text': translate['options']['string'] }))
    translate['stdout'] = ''
    translate['hook_process'] = weechat.hook_process_hashtable('url:%s' % url,
                                                               { 'useragent': 'Mozilla/5.0' },
                                                               10 * 1000,
                                                               'translate_process_cb', '')

    return weechat.WEECHAT_RC_OK

if __name__ == '__main__' and import_ok:
    if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION,
                        SCRIPT_LICENSE, SCRIPT_DESC,
                        '', ''):
        # set default settings
        for option, default_value in translate_settings.items():
            if not weechat.config_is_set_plugin(option):
                weechat.config_set_plugin(option, default_value)
        # replace old URL (not working any more) by new one
        if weechat.config_get_plugin('url') == 'http://ajax.googleapis.com/ajax/services/language/translate':
            weechat.config_set_plugin('url', translate_settings['url'])
            weechat.prnt('', 'translate: default URL has been set (to replace old URL)')
        # new command
        marker = weechat.config_get_plugin('marker')
        weechat.hook_command('translate',
                             'Translate string using Google translate API.',
                             '[from_to | !] [< | + | text] || -keys',
                             ' from: base language\n'
                             '   to: target language\n'
                             '    !: invert languages (translate from "to" to "from")\n'
                             '    <: restore input as it was before last translation\n'
                             '       (if you do it again, it will restore input after translation)\n'
                             '    +: translate only last word\n'
                             ' text: translate this text\n'
                             '       (if no text is given, input is used, useful when this command is bound to a key)\n'
                             '-keys: create some default keys (if some keys already exist, they are NOT overwritten):\n'
                             '         meta-tmeta-t => /translate    (translate using default from_to)\n'
                             '         meta-tmeta-r => /translate !  (reverse of from_to)\n'
                             '         meta-tmeta-w => /translate +  (translate last word)\n'
                             '         meta-tmeta-u => /translate <  (restore input)\n\n'
                             'You can define alias for /translate:\n'
                             '  /alias tr /translate\n\n'
                             'Note 1: when translating command line, you can start with "from_to" or "!".\n\n'
                             'Note 2: input can contain a marker ("%s"), translation will start after this marker '
                             '(beginning of input will not be translated).\n'
                             'Example: "this text &&est traduit" => "this text is translated"\n\n'
                             'To define default languages, for example english to german:\n'
                             '  /set plugins.var.python.translate.default "en_de"\n\n'
                             'Examples:\n'
                             '  /translate ! this is a test\n'
                             '  /translate en_it I want this string in italian' % marker,
                             '-keys|!|<|+', 'translate_cmd_cb', '')



# pagetitle plugin for weechat-0.3.0
#
#  /pt http://tech.slashdot.org/tech/08/11/12/199215.shtml
#  <user> http://tech.slashdot.org/tech/08/11/12/199215.shtml
#		 ('Slashdot | Microsoft's "Dead Cow" Patch Was 7 Years In the Making')
#
# xororand @ irc://irc.freenode.net/#weechat
#
# 2009-05-02, FlashCode <flashcode@flashtux.org>:
#     version 0.5: sync with last API changes

import htmllib
import re
import socket
import sys
import urllib2

limit_title_length = 100
debug = True

# user agent
opener = urllib2.build_opener()
opener.addheaders = [('User-agent', 'Mozilla/5.0 (weechat/pagetitle)')]
urllib2._urlopener = opener

# set a short timeout to avoid freezing weechat [seconds]
socket.setdefaulttimeout(5)

regex_url = re.compile("""https?://[^ ]+""")

def unescape(s): #{{{
	"""Unescape HTML entities"""
	p = htmllib.HTMLParser(None)
	p.save_bgn()
	p.feed(s)
	return p.save_end() #}}}

def getPageTitle(url):
	"""Retrieve the HTML <title> from a webpage"""

	try:
		u = urllib2.urlopen(url)
	except urllib2.HTTPError, e:
		raise NameError(str(e))
	except urllib2.URLError, e:
		raise NameError(str(e))

	info = u.info()
	try:
		content_type = info['Content-Type']
		if not re.match(".*/html.*",content_type):
			return ""
	except:
		return ""

	head = u.read(8192)
	head = re.sub("[\r\n\t ]"," ",head)

	title = re.search('(?i)\<title\>(.*?)\</title\>', head)
	if title:
		title = title.group(1)
		return unescape(title)
	else:
		return ""

# /pt http://foo
def on_pagetitle(data, buffer, args):
	if len(args) == 0:
		return weechat.WEECHAT_RC_ERROR

	msg = args

	def urlReplace(match):
		url = match.group()
		try:
			if debug:
				weechat.prnt(buffer, "pagetitle: retrieving '%s'" % url)
			title = getPageTitle(url)
			if len(title) > limit_title_length:
				title = "%s [...]" % title[0:limit_title_length]
			url = "%s ('%s')" % (url, title)
		except NameError, e:
			weechat.prnt(buffer, "pagetitle: URL: '%s', Error: '%s'" % (url, e))
		return url

	msg = regex_url.sub(urlReplace, msg)
	weechat.command(buffer, "/say %s" % msg)

	return weechat.WEECHAT_RC_OK

# Register plugin
import weechat

weechat.register ('pagetitle', 'xororand', '0.5', 'GPL3', """Adds HTML titles to http:// urls in your message.""", "", "")
desc = """Sends a message to the current buffer and adds HTML titles to http:// URLs.
Example: /pt check this out: http://xkcd.com/364/
<you> check this out: http://xkcd.com/364/ (xkcd - A webcomic of romance, sarcasm, math and language)"""
weechat.hook_command ('pt', desc, 'message', 'message containing an URL', '', 'on_pagetitle', '')

# vim:set ts=4 sw=4 noexpandtab nowrap foldmethod=marker:




"""
  Author:
    Pablo Escobar <pablo__escobar AT tlen DOT pl>

  Adapted for Weechat 0.3.0 by:
    Apprentice <apprent1ce AT livejournal DOT com>

  What it does:
    This script shows the currently played song in mpd 

  Usage:
    /weempd - Displays the songname 

  Released under GNU GPL v3 or newer
"""

import weechat as wc
import re
from os.path import basename, splitext
from os import popen

wc.register("weempd", "Apprentice", "0.1.1", "GPL3", "np for mpd", "", "")

def subst(text, values):
  out = ""
  n = 0
  for match in re.finditer(findvar, text):
    if match is None:
      continue 
    else:
      l, r = match.span()
      nam = match.group(1)
      out += text[n:l+1] + values.get(nam, "") #"$" + nam)
      n = r
  return out + text[n:]


def np(data, buffer, args):
  """
   Send information about the currently
   played song to the channel.
  """
  spacer = wc.config_get_plugin("spacer")
  msg_head = wc.config_get_plugin("msg_head")
  tempinfo = popen('mpc').readline().rstrip()
  if tempinfo.find("volume:") == -1:
    all = '/me ' + msg_head + spacer + tempinfo 
    wc.command(wc.current_buffer(), all)
  return 0
  
wc.hook_command("weempd", "now playing", "", np.__doc__, "", "np", "")

default = {
  "msg_head": "np:",
  "spacer": " " ,
}

for k, v in default.items():
  if not wc.config_is_set_plugin(k):
    wc.config_set_plugin(k, v)



# -*- coding: utf-8 -*-
# Author: sitaktif <romainchossart AT gmail DOT com>
# This plugin calls the pynma bindings via python when somebody says your
# nickname, sends you a query, etc.
#
# Requires:
# Weechat 0.3.0
# pynma.py (NMA python bindings) - This is the python API for NMA.  Take it
#   from https://raw.github.com/sitaktif/weechat_plugins_nma/master/pynma.py
#   and put it in the same direactory as nma.py.
#   Note: the pynma.py script is taken from https://github.com/uskr/pynma and
#     is not modified. The only reason you should use the one in the weechat
#     plugin repo (as opposed to the pynma repo) is that you are guaranteed
#     that pynma.py is at a version that is compatible with the weechat plugin.
#
# License: Released under GNU GPL v2
#
# Acknowledgements: Based on lavaramano's script "notify.py" v. 0.0.5 (thanks!)
#
# 2014-05-10, Sébastien Helleu <flashcode@flashtux.org>
#     version 1.0.7: change hook_print callback argument type of
#                    displayed/highlight (WeeChat >= 1.0)
# 2013-10-06, sitaktif
#     version 1.0.6: Add a "test" action. Take API key config changes dynamically.
# 2012-09-24, sitaktif
#     version 1.0.5: Do not send a notification when one is *sending* a query
#     message.
# 2012-05-06, sitaktif
#     version 1.0.4: Add an option to send everything in push. Also change
#     default delimiters with brackets and make lines 80 chars wide.
# 2012-05-05, sitaktif
#     version 1.0.3: Manage (non-ASCII) UTF8 chars
# 2012-01-05, Ac-town
#     version 1.0.2: Fixes a few typos I ran into and adds only_away. Only_away
#     only sends notifications if you are marked away.
# 2011-09-19, sitaktif
#     version 1.0.1: Corrected a bug with debug functions
# 2011-07-22, sitaktif
#     version 1.0.0: Initial release

import re
import weechat as w

w.register("nma", "sitaktif", "1.0.7", "GPL2",
    "nma: Receive notifications on NotifyMyAndroid app.", "", "")

# script options
settings = {
    "apikey"               : ("",      "Your NMA API key"),
    "nick_separator_left"  : ("(",     "Left separator for the nick that highlighted you"),
    "nick_separator_right" : (") ",    "Right separator for the nick that highlighted you"),
    "emergency_hilights"   : ("-1",    "Emergency of the highlight notifications (-2 is lowest, 2 is highest)"),
    "emergency_priv_msg"   : ("0",     "Emergency of the query notifications (-2 is lowest, 2 is highest)"),
    "activated"            : ("on",    "Whether the plugin will send notifications or not"),
    "notify_hilights"      : ("on",    "Send NMA notifications when you get highlights"),
    "notify_priv_msg"      : ("on",    "Send NMA notifications when you get a query message"),
    "use_push_if_possible" : ("on",    "If on, will try to fit the whole message in the title, which is send with the PUSH protocol. This makes you receive queries more quickly."),
    "smart_notification"   : ("off",   "Don't send notifications if you are focusing the channel (default: off)"),
    "only_away"            : ("off",   "Only send notifications if you are away"),
    "debug"                : ("off",   "Print debug messages"),
}

#severity_t = {
#    "emergency" : 2,
#    "high" : 1,
#    "normal" : 0,
#    "moderate" : -1,
#    "low": -2
#}

"""
Init
"""

for option, (default_value, description) in settings.items():
    if w.config_get_plugin(option) == "":
        w.config_set_plugin(option, default_value)
    if description:
        w.config_set_desc_plugin(option, description)

if w.config_get_plugin("apikey") == "":
    w.prnt("", "[nma] - You haven't set your API key. Use /set "
            "plugins.var.python.nma.apikey \"you_nma_api_token\" "
            "to fix that.")


"""
Hooks
"""

# Hook command
w.hook_command("nma", "Activate NotifyMyAndroid notifications",
        "on | off | test",
        """    on : Activate notifications
    off : Deactivate notifications
    test : Test a notification\n
        """,
        "on || off || test",
        "nma_cmd_cb", "");
# Hook privmsg/hilights
w.hook_print("", "irc_privmsg", "", 1, "priv_msg_cb", "")

from pynma import PyNMA
p = PyNMA()

"""
Helpers
"""

def _debug(text):
    if w.config_string_to_boolean(w.config_get_plugin("debug")):
        w.prnt("", "[nma] - %s" % text)

def nma_get_instance():
    global p
    for k in p._apikey:
        p.delkey(k)
    key = w.config_get_plugin("apikey")
    if key:
        p.addkey(key)
    else:
        w.prnt("", "[nma] - Cannot do anyting without an API key.")
        return None
    return p


"""
Functions
"""

# /nma command callback. Arguments: bool (on/off)
def nma_cmd_cb(data, buffer, args):
    if args.strip() == "test":
        send_notification("This is a weechat nma plugin test",
                "If you receive that, your configuration is ok", 0)
        return w.WEECHAT_RC_OK

    bool_arg = w.config_string_to_boolean(args)
    status = "%sactivated" % ("" if bool_arg else "de")
    ret = w.config_set_plugin('activated', args)
    if ret == w.WEECHAT_CONFIG_OPTION_SET_OK_SAME_VALUE:
        w.prnt("", "...NMA was already %s" % status)
    elif ret == w.WEECHAT_CONFIG_OPTION_SET_ERROR:
        w.prnt("", "Error while setting the config.")
        return w.WEECHAT_RC_ERROR
    else:
        w.prnt("", "Notify My Android notifications %s." % status)
    return w.WEECHAT_RC_OK


def priv_msg_cb(data, bufferp, uber_empty, tagsn, isdisplayed,
        ishilight, prefix, message):
    """Sends highlighted message to be printed on notification"""

    if not w.config_string_to_boolean(w.config_get_plugin('activated')):
        _debug('Plugin not activated. Not sending.')
        return w.WEECHAT_RC_OK

    if (w.config_string_to_boolean(w.config_get_plugin('smart_notification')) and
            bufferp == w.current_buffer()):
        _debug('"smart_notification" option set but you are on this buffer already. Not sending.')
        return w.WEECHAT_RC_OK

    if (w.config_string_to_boolean(w.config_get_plugin('only_away')) and not
            w.buffer_get_string(bufferp, 'localvar_away')):
        _debug('"only_away" option set but you are not away. Not sending.')
        return w.WEECHAT_RC_OK

    notif_body = u"%s%s%s%s" % (
            w.config_get_plugin('nick_separator_left').decode('utf-8'),
            prefix.decode('utf-8'),
            w.config_get_plugin('nick_separator_right').decode('utf-8'),
            message.decode('utf-8'))


    # Check that it's in a "/q" buffer and that I'm not the one writing the msg
    is_pm = w.buffer_get_string(bufferp, "localvar_type") == "private"
    is_notify_private = re.search(r'(^|,)notify_private(,|$)', tagsn) is not None
    # PM (query)
    if (is_pm and is_notify_private and
            w.config_string_to_boolean(w.config_get_plugin('notify_priv_msg'))):
        send_notification("IRC private message",
        notif_body, int(w.config_get_plugin("emergency_priv_msg")))


    # Highlight (your nick is quoted)
    elif (int(ishilight) and
            w.config_string_to_boolean(w.config_get_plugin('notify_hilights'))):
        bufname = (w.buffer_get_string(bufferp, "short_name") or
                w.buffer_get_string(bufferp, "name"))
        send_notification(bufname.decode('utf-8'), notif_body,
                int(w.config_get_plugin("emergency_hilights")))

    return w.WEECHAT_RC_OK


def send_notification(chan, message, priority):
    p = nma_get_instance()
    if not(p):
        w.prnt("", "[nma] - Problem with NMA instance. Not sending notification")
        return None

    _debug("Sending notif with chan '%s', message '%s' and priority '%s'." %
            (chan, message, priority))
    if w.config_string_to_boolean(w.config_get_plugin('use_push_if_possible')):
        # So far, the length is hardcoded in pynma.py...
        if len(chan) + len(message) < 1021:
            chan = "%s - %s" % (chan, message)
            message = ""
    ret = p.push("[IRC]", chan, message, '', priority, batch_mode=False)
    if ret is not None:
        _debug("Message sent: %s. Return: %s." % (message, ret))

# vim: autoindent expandtab smarttab shiftwidth=4



# -*- coding: utf-8 -*-
#
# Copyright (C) 2010 Chmouel Boudjnah <chmouel@chmouel.com>
# Copyright (C) 2012-2013 bwidawsk <ben@bwidawsk.net>
# License: GPL3
#
# plugin to get alternate timezones in a weechat bar
#
# Changelog:
#  0.2 Added help, and multiple timezeones
#  0.1 first version
#

import weechat as w
import pytz
import datetime

SCRIPT_NAME    = "alternatetz"
SCRIPT_AUTHOR  = "Chmouel Boudjnah <chmouel@chmouel.com>"
SCRIPT_VERSION = "0.2"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC    = "Display Alternate Time from different Timezones"

SCRIPT_COMMAND = 'alternatetz'

OPTIONS		= {
'timezone'	: ('GMT', 'list of timezones to display. The list is comprised of space separated list timezones using the Olson tz database'),
'timeformat'	: ('%H:%M', 'strftime compatible format')
}

def alternatetz_item_cb(*kwargs):
    ret = ''
    tznames = OPTIONS['timezone'].split()
    for tzname in tznames:
        tz = pytz.timezone(tzname)
	ret += tz.zone + ': ' + datetime.datetime.now(tz).strftime(OPTIONS['timeformat']) + ' '
    return ret[:-1]

def alternatetz_timer_cb(*kwargs):
    w.bar_item_update('alternatetz')
    return w.WEECHAT_RC_OK

if __name__ == '__main__':
    w.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, '', '')
    for option,value in list(OPTIONS.items()):
        w.config_set_desc_plugin(option, '%s (default: "%s")' % (value[1], value[0]))
        if not w.config_is_set_plugin(option):
            w.config_set_plugin(option, value[0])
            OPTIONS[option] = value[0]
        else:
            OPTIONS[option] = w.config_get_plugin(option)

    w.bar_item_new('alternatetz', 'alternatetz_item_cb', '')
    w.bar_item_update('alternatetz')
    w.hook_timer(1000*60, 60, 0, 'alternatetz_timer_cb', '')

# vim: tabstop=8 expandtab shiftwidth=4 softtabstop=4



# -*- coding: utf-8 -*-
#
# Copyright (c) 2009 by xt <xt@bash.no>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

#
# (this script requires WeeChat 0.3.0 or newer)
#
# History:
# 2016-02-05, ixti
#  version 0.4: Add Python3 support
# 2009-12-15, xt
#  version 0.3: moved around some control structures to not be as noisy
# 2009-12-02, xt
#  version 0.2: bugfix, more printing
# 2009-12-01, xt <xt@bash.no>
#  version 0.1: initial release

import weechat as w
import time

SCRIPT_NAME    = "buffer_autoclose"
SCRIPT_AUTHOR  = "xt <xt@bash.no>"
SCRIPT_VERSION = "0.4"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC    = "Automatically close inactive private message buffers"

settings = {
        'interval': '1', # How often in minutes to check
        'age_limit': '30', # How old in minutes before auto close
        'ignore': '', # Buffers to ignore (use full name: server.buffer_name)
}

if w.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE,
                    SCRIPT_DESC, "", ""):
    for option, default_value in settings.items():
        if not w.config_is_set_plugin(option):
            w.config_set_plugin(option, default_value)
    w.hook_timer(\
            int(w.config_get_plugin('interval')) * 1000 * 60,
            0,
            0,
            "close_time_cb",
            '')


def get_all_buffers():
    '''Returns list with pointers of all open buffers.'''
    buffers = []
    infolist = w.infolist_get('buffer', '', '')
    while w.infolist_next(infolist):
        buffer_type = w.buffer_get_string(w.infolist_pointer(infolist, 'pointer'), 'localvar_type')
        if buffer_type == 'private': # we only close private message buffers for now
            buffers.append(w.infolist_pointer(infolist, 'pointer'))
    w.infolist_free(infolist)
    return buffers

def get_last_line_date(buffer):
    date = '1970-01-01 01:00:00'
    infolist = w.infolist_get('buffer_lines', buffer, '')
    while w.infolist_prev(infolist):
        date = w.infolist_time(infolist, 'date')
        if date != '1970-01-01 01:00:00':
        # Some lines like "Day changed to" message doesn't have date 
        # set so loop until we find a message that does
            break
    w.infolist_free(infolist)
    return date

def is_in_hotlist(buffer):
    ''' Returns true if buffer is in hotlist, false if not'''

    hotlist = w.infolist_get('hotlist', '', '')
    found = False
    while w.infolist_next(hotlist):
        thebuffer = w.infolist_pointer(hotlist, 'buffer_pointer')
        if thebuffer == buffer:
            found = True
            name = w.buffer_get_string(thebuffer, 'short_name')
            break

    w.infolist_free(hotlist)
    return found

def close_time_cb(buffer, args):
    ''' Callback for check for inactivity and close '''

    for buffer in get_all_buffers():
        name = w.buffer_get_string(buffer, 'name')


        date = get_last_line_date(buffer)
        date = time.mktime(time.strptime(date, '%Y-%m-%d %H:%M:%S'))
        now = time.time()
        seconds_old = now - date
        if seconds_old > int(w.config_get_plugin('age_limit'))*60:
            if is_in_hotlist(buffer):
                #w.prnt('', '%s: Not closing buffer: %s: it is in hotlist' %(SCRIPT_NAME, name))
                continue
            if name in w.config_get_plugin('ignore').split(','):
                #w.prnt('', '%s: Not closing buffer: %s: it is in ignore list' %(SCRIPT_NAME, name))
                continue
            if buffer == w.current_buffer():
                # Never close current buffer
                #w.prnt('', '%s: Not closing buffer: %s: it is in currently active' %(SCRIPT_NAME, name))
                continue
            if len(w.buffer_get_string(buffer, 'input')):
                # Don't close buffers with text on input line
                #w.prnt('', '%s: Not closing buffer: %s: it has input' %(SCRIPT_NAME, name))
                continue

            w.prnt('', '%s: Closing buffer: %s' %(SCRIPT_NAME, name))
            w.command(buffer, '/buffer close')
        #else:
        #    w.prnt('', '%s: Not closing buffer: %s: it is too new: %s' %(SCRIPT_NAME, name, seconds_old))

    return w.WEECHAT_RC_OK



# This script sends "uname -a" output to current channel.
# 	Just type /uname while chatting on some channel ;)
#
# 		by Stalwart <stlwrt doggy gmail.com>
#
# port to WeeChat 0.3.0 by Benjamin Neff (SuperTux88) <info@benjaminneff.ch>
#
# Released under GPL licence.


SCRIPT_NAME    = "uname"
SCRIPT_AUTHOR  = "Stalwart <stlwrt doggy gmail.com>"
SCRIPT_VERSION = "1.1"
SCRIPT_LICENSE = "GPL2"
SCRIPT_DESC    = "Sends \"uname -a\" output to current channel"

import_ok = True

try:
    import weechat
except ImportError:
    print "This script must be run under WeeChat."
    print "Get WeeChat now at: http://www.weechat.org/"
    import_ok = False

try:
    from os import popen
except ImportError, message:
    print "Missing package(s) for %s: %s" % (SCRIPT_NAME, message)
    import_ok = False

def senduname(data, buffer, args):
	unameout = popen ('uname -a')
	uname = unameout.readline()
	weechat.command(buffer, "uname -a: " + uname[:-1])
	return weechat.WEECHAT_RC_OK

if __name__ == "__main__" and import_ok:
	if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, "", ""):
		weechat.hook_command (SCRIPT_NAME, SCRIPT_DESC, '','Just type /uname while chatting on some channel ;)','', 'senduname', '')



# -*- coding: utf-8 -*-
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import weechat as w

SCRIPT_NAME = "buffer_bind"
SCRIPT_AUTHOR = "Trevor 'tee' Slocum <tslocum@gmail.com>"
SCRIPT_VERSION = "1.0"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC = "Bind meta-<key> to the current buffer"
SCRIPT_NOTE = """Case sensitivity is controlled via plugins.var.python.%s.case_sensitive (default: off)

%s is a port of irssi's window_alias written by veli@piipiip.net""" % (SCRIPT_NAME, SCRIPT_NAME)

SETTINGS = {
    "case_sensitive": "off"
}


def command_buffer_bind(data, buffer, args):
    if len(args) == 1 and args[0] != "":
        bindkey = args[0]
        buffername = w.buffer_get_string(buffer, "name")

        bind_keys = [bindkey]
        if w.config_get_plugin("case_sensitive") == "off" and bindkey.isalpha():
            bind_keys.append(bindkey.swapcase())
        for bind_keys_i in bind_keys:
            w.command(buffer, "/key bind meta-%s /buffer %s" % (bind_keys_i, buffername))

        w.prnt(buffer, "Buffer %s is now accessible with meta-%s" % (buffername, bindkey))
    else:
        w.command(buffer, "/help %s" % SCRIPT_NAME)

    return w.WEECHAT_RC_OK_EAT


if w.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE,
              SCRIPT_DESC, "", ""):
    for option, value in SETTINGS.items():
        if not w.config_is_set_plugin(option):
            w.config_set_plugin(option, value)

    w.hook_command(SCRIPT_NAME, SCRIPT_DESC, "<key>", SCRIPT_NOTE, "key", "command_buffer_bind", "")



# -*- coding: utf-8 -*-
###
# Copyright (c) 2009-2010 by Elián Hanisch <lambdae2@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###

###
# Notifications for WeeChat
#
#   Notification script that uses libnotify or dbus, supports WeeChat inside screen.
#   Uses a xmlrpc daemon that must be running in the receiving machine (remotely or locally)
#
#   The daemon can be setup in several ways, see inotify-daemon --help
#   Download it from 'http://github.com/m4v/inotify-daemon/raw/stable/inotify-daemon'
#
#   Commands:
#   * /inotify
#     See /help inotify
#
#   Settings:
#   * plugins.var.python.inotify.server_uri:
#   inotify-daemon address and port to connect, must be the same address the daemon is using.
#   By default it uses localhost and port 7766.
#
#       Examples:
#       http://www.your.home.com:7766
#       http://localhost:7766
#
#   * plugins.var.python.inotify.server_method:
#   Notification method supported by the daemon to use. Defaults to 'libnotify'.
#   See below for detailed help about them.
#
#   * plugins.var.python.inotify.color_nick:
#   Will use coloured nicks in notifications.
#
#   * plugins.var.python.inotify.ignore_channel:
#   Comma separated list of patterns for define ignores. Notifications from channels where its name
#   matches any of these patterns will be ignored.
#   Wildcards '*', '?' and char groups [..] can be used.
#   An ignore exception can be added by prefixing '!' in the pattern.
#
#       Example:
#       *ubuntu*,!#ubuntu-offtopic
#       any notifications from a 'ubuntu' channel will be ignored, except from #ubuntu-offtopic
#
#   * plugins.var.python.inotify.ignore_nick:
#   Same as ignore_channel, but for nicknames.
#
#       Example:
#       troll,b[0o]t
#       will ignore notifications from troll, bot and b0t
#
#   * plugins.var.python.inotify.ignore_text:
#   Same as ignore_channel, but for the contents of the message.
#
#   * plugins.var.python.inotify.passwd:
#   In the case the daemon is using a password for verify that incoming notifications are trusted.
#   If this password doesn't match with the password setup in inotify-daemon notification will not
#   succeed.
#
#
#   Notify methods:
#   * libnotify:
#   Use libnotify for notifications, needs python-notify installed in the machine running the
#   daemon. This is the default method.
#
#   * dbus:
#   Uses dbus directly for notifications, this is KDE4 specific, might not work in other desktops.
#   Needs python-dbus in the machine running the daemon.
#
#   * any:
#   Use daemon's configured method, this is usually libnotify.
#
#
#   TODO
#   add commands for configure ignores
#   add more notifications methods (?)
#
#
#   History:
#   2014-05-10, Sébastien Helleu <flashcode@flashtux.org>
#   version 0.1.4: change hook_print callback argument type of
#                  displayed/highlight (WeeChat >= 1.0)
#   2011-11-02, Sebastien Helleu <flashcode@flashtux.org>:
#   version 0.1.3: use local variable "channel" in buffer instead of reading "short_name",
#                  fix command for hook_process (remove line break before "-c")
#
#   2011-03-11, Sebastien Helleu <flashcode@flashtux.org>:
#   version 0.1.2: get python 2.x binary for hook_process (fix problem when
#                  python 3.x is default python version)
#   2010-03-10:
#   version 0.1.1: fixes
#   * improved shell escapes when using hook_process
#   * fix ACTION messages
#
#   2010-02-24
#   version 0.1: release!
#
###

SCRIPT_NAME    = "inotify"
SCRIPT_AUTHOR  = "Elián Hanisch <lambdae2@gmail.com>"
SCRIPT_VERSION = "0.1.4"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC    = "Notifications for WeeChat."
SCRIPT_COMMAND = "inotify"

DAEMON_URL = 'http://github.com/m4v/inotify-daemon/raw/stable/inotify-daemon'
DAEMON     = 'inotify-daemon'
DAEMON_VERSION = '0.2'

### Default Settings ###
settings = {
'server_uri'     : 'http://localhost:7766',
'server_method'  : 'any',
'color_nick'     : 'on',
'ignore_channel' : '',
'ignore_nick'    : '',
'ignore_text'    : '',
'passwd'         : '',
}

max_error_count = 3

try:
    import weechat
    WEECHAT_RC_OK = weechat.WEECHAT_RC_OK
    import_ok = True
except:
    print "This script must be run under WeeChat."
    print "Get WeeChat now at: http://www.weechat.org/"
    import_ok = False

import xmlrpclib, socket
from fnmatch import fnmatch

# remote daemon timeout
socket.setdefaulttimeout(4)

### Messages ###
def debug(s, prefix=''):
    """Debug msg"""
    if not weechat.config_get_plugin('debug'): return
    buffer_name = 'DEBUG_' + SCRIPT_NAME
    buffer = weechat.buffer_search('python', buffer_name)
    if not buffer:
        buffer = weechat.buffer_new(buffer_name, '', '', '', '')
        weechat.buffer_set(buffer, 'nicklist', '0')
        weechat.buffer_set(buffer, 'time_for_each_line', '0')
        weechat.buffer_set(buffer, 'localvar_set_no_log', '1')
    weechat.prnt(buffer, '%s\t%s' %(prefix, s))

def error(s, prefix='', buffer='', trace=''):
    """Error msg"""
    if weechat.config_get_plugin('quiet'): return
    prefix = prefix or script_nick
    weechat.prnt(buffer, '%s%s %s' %(weechat.prefix('error'), prefix, s))
    if weechat.config_get_plugin('debug'):
        if not trace:
            import traceback
            if traceback.sys.exc_type:
                trace = traceback.format_exc()
        not trace or weechat.prnt('', trace)

def say(s, prefix='', buffer=''):
    """normal msg"""
    prefix = prefix or script_nick
    weechat.prnt(buffer, '%s\t%s' %(prefix, s))

### Config and value validation ###
boolDict = {'on':True, 'off':False}
def get_config_boolean(config):
    value = weechat.config_get_plugin(config)
    try:
        return boolDict[value]
    except KeyError:
        default = settings[config]
        error("Error while fetching config '%s'. Using default value '%s'." %(config, default))
        error("'%s' is invalid, allowed: 'on', 'off'" %value)
        return boolDict[default]

def get_config_int(config, allow_empty_string=False):
    value = weechat.config_get_plugin(config)
    try:
        return int(value)
    except ValueError:
        if value == '' and allow_empty_string:
            return value
        default = settings[config]
        error("Error while fetching config '%s'. Using default value '%s'." %(config, default))
        error("'%s' is not a number." %value)
        return int(default)

valid_methods = set(('any', 'dbus', 'libnotify'))
def get_config_valid_string(config, valid_strings=valid_methods):
    value = weechat.config_get_plugin(config)
    if value not in valid_strings:
        default = settings[config]
        error("Error while fetching config '%s'. Using default value '%s'." %(config, default))
        error("'%s' is an invalid value, allowed: %s." %(value, ', '.join(valid_strings)))
        return default
    return value

### Class definitions ###
class Ignores(object):
    def __init__(self, ignore_type):
        self.ignore_type = ignore_type
        self.ignores = []
        self.exceptions = []
        self._get_ignores()

    def _get_ignores(self):
        assert self.ignore_type is not None
        ignores = weechat.config_get_plugin(self.ignore_type).split(',')
        ignores = [ s.lower() for s in ignores if s ]
        self.ignores = [ s for s in ignores if s[0] != '!' ]
        self.exceptions = [ s[1:] for s in ignores if s[0] == '!' ]

    def __contains__(self, s):
        s = s.lower()
        for p in self.ignores:
            if fnmatch(s, p):
                for e in self.exceptions:
                    if fnmatch(s, e):
                        return False
                return True
        return False


class Server(object):
    def catch_exceptions(f):
        """
        This decorator is for catch exceptions in methods that communicate with the daemon."""
        def protected_method(self, *args):
            try:
                return f(self, *args)
            except xmlrpclib.Fault, e:
                self._error("A fault occurred." %e, trace="Code: %s\nString: %s" %(e.faultCode, e.faultString))
            except xmlrpclib.ProtocolError, e:
                self._error("Protocol error: %s" %e, trace="Url: %s\nCode: %s\nHeaders: %s\nMessage: %s" %(e.url, e.errcode,
                    e.headers, e.errmsg))
            except socket.error, e:
                self._error_connect()
            except socket.timeout, e:
                self._error('Timeout while sending to our daemon.')
                if self.error_count < 3: # don't re-queue after 3 errors
                    return 'retry'
            except:
                # catch all exception
                self._error("An error occurred, but I'm not sure what could it be...")
        return protected_method

    def __init__(self):
        self._reset()
        self._create_server()
        if not self.error_count:
            self.send_rpc('Notification script loaded')

    def _reset(self):
        self.msg = {}
        self.timer = None

    def enqueue(self, msg, channel):
        self._enqueue(msg, channel)

    def _enqueue(self, msg, channel='', timeout=3000):
        if channel not in self.msg:
            self.msg[channel] = msg
        else:
            s = self.msg[channel]
            msg = '%s\n%s' %(s, msg)
            self.msg[channel] = msg
        if self.timer is None:
            self.timer = weechat.hook_timer(timeout, 0, 1, 'msg_flush', '')
            #debug('set timer: %s %s' %(self.timer, timeout))

    def flush(self):
        for channel, msg in self.msg.iteritems():
            if self.send_rpc(msg, channel) == 'retry':
                # daemon is restarting, try again later
                self._restart_timer()
                return
        if self.remote:
            #  we can't stop flushing if we're in remote mode, so save a copy as we might need
            #  to repeat the queue later
            self.msg_bak = self.msg.copy()
        self._reset()

    def _restart_timer(self):
        if self.timer is not None:
            #debug('reset and set timer')
            weechat.unhook(self.timer)
        self.timer = weechat.hook_timer(5000, 0, 1, 'msg_flush', '')

    @catch_exceptions
    def _create_server(self):
        self.error_count = 0
        self.method = get_config_valid_string('server_method')
        self.address = weechat.config_get_plugin('server_uri')
        # detect if we're going to connect to localhost.
        if self.address[:17] in ('http://localhost:', 'http://127.0.0.1:'):
            self.remote = False
        else:
            self.remote = True
            self.msg_bak = {}
        self.server = xmlrpclib.Server(self.address)
        version = self.server.version()
        if version != DAEMON_VERSION:
            error('Incorrect daemon version, should be %s, but got %s' %(DAEMON_VERSION,
                version))
            error('Download the latest %s from %s' %(DAEMON, DAEMON_URL))

    def _error(self, s, **kwargs):
        if self.error_count < max_error_count: # stop sending error msg after max reached
            error(s, **kwargs)
        elif self.error_count == max_error_count:
            error('Suppressing future error messages...')
        self.error_count += 1

    def _error_connect(self):
        self._error('Failed to connect to our notification daemon, check if the address'
               ' \'%s\' is correct and if it\'s running.' %self.address)

    @catch_exceptions
    def send_rpc(self, *args):
        debug('sending rpc: %s' %' '.join(map(repr, args)))
        passwd = weechat.config_get_plugin('passwd')
        if self.remote:
            return self._send_rpc_process(passwd, *args)
        rt = getattr(self.server, self.method)(passwd, *args)
        if rt == 'OK':
            self.error_count = 0 
            #debug('Success: %s' % rt)
        elif rt.startswith('warning:'):
            self._error(rt[8:])
            if self.error_count < 10: # don't re-queue after 10 errors
                #debug('repeating queue')
                # returning 'retry' will cause flush() to try to send msgs again later
                return 'retry'
        else:
            error(rt)

    def _send_rpc_process(self, *args):
        def quoted(s):
            """
            Is important to escape quotes properly so hook_process doesn't break or sends stuff
            outside single quotes."""
            if '\\' in s:
                # escape any backslashes
                s = s.replace('\\', '\\\\')
            if '"' in s:
                s = s.replace('"', '\\"')
            if "'" in s:
                # I must escape single quotes with \'\\\'\' because they will be within single
                # quotes in the command string. Awesome.
                s = s.replace("'", "'\\''")
            return  '"""%s"""' %s

        args = ', '.join(map(quoted, args))
        python2_bin = weechat.info_get('python2_bin', '') or 'python'
        cmd = python2_bin + rpc_process_cmd %{'server_uri':self.address, 'method':self.method, 'args':args}
        debug('\nRemote cmd:%s\n' %cmd)
        weechat.hook_process(cmd, 30000, 'rpc_process_cb', '')

    @catch_exceptions
    def quit(self):
        passwd = weechat.config_get_plugin('passwd')
        rt = self.server.quit(passwd)
        debug(rt)
        if rt != 'OK':
            error(rt)

    @catch_exceptions
    def restart(self):
        passwd = weechat.config_get_plugin('passwd')
        rt = self.server.restart(passwd)
        debug(rt)
        if rt != 'OK':
            error(rt)


### Functions ###
def msg_flush(*args):
    server.flush()
    return WEECHAT_RC_OK

# command MUST be within single quotes, otherwise the shell would try to expand stuff and it might
# be real nasty, somebody could run arbitrary code with a highlight.
rpc_process_cmd = """ -c '
import xmlrpclib
try:
    server = xmlrpclib.Server("%(server_uri)s")
    print getattr(server, "%(method)s")(%(args)s)
except Exception, e:
    print "error: %%s" %%e'
"""

def rpc_process_cb(data, command, rc, stdout, stderr):
    #debug("%s\nstderr: %s\nstdout: %s" %(rc, repr(stderr), repr(stdout)))
    if stdout:
        debug('Reply: %s' %stdout)
        if stdout == 'OK\n':
            server.error_count = 0
        elif stdout.startswith('warning:'):
            server._error(stdout[8:])
            if server.error_count < 10:
                server.msg = server.msg_bak
                server._restart_timer()
        else:
            server._error(stdout)
    if stderr:
        error(stderr)
    return WEECHAT_RC_OK

color_table = ('teal', 'darkmagenta', 'darkgreen', 'brown', 'blue', 'darkblue', 'darkcyan', 'magenta', 'green', 'grey')

def color_tag(nick):
    n = len(color_table)
    #generic_nick = nick.strip('_`').lower()
    id = (sum(map(ord, nick))%n)
    #debug('%s:%s' %(nick, id))
    return '<font color=%s>&lt;%s&gt;</font>' %(color_table[id], nick)

def format(s, nick=''):
    if '<' in s:
        s = s.replace('<', '&lt;')
    if '>' in s:
        s = s.replace('>', '&gt;')
    if '"' in s:
        s = s.replace('"', '&quot;')
    if '\n' in s:
        s = s.replace('\n', '<br/>')
    if nick:
        if get_config_boolean('color_nick'):
            nick = color_tag(nick)
        else:
            nick = '&lt;%s&gt;' %nick
        s = '<b>%s</b> %s' %(nick, s)
    return s

def send_notify(s, channel='', nick=''):
    #command = getattr(server, 'kde4')
    s = format(s, nick)
    server.enqueue(s, channel)

def is_displayed(buffer):
    """Returns True if buffer is in a window and the user is active. This is for not show
    notifications of a visible buffer while the user is doing something and wouldn't need to be
    notified."""
    window = weechat.buffer_get_integer(buffer, 'num_displayed')
    if window != 0:
        return not inactive()
    return False

def inactive():
    inactivity = int(weechat.info_get('inactivity', ''))
    #debug('user inactivity: %s' %inactivity)
    if inactivity > 20:
        return True
    else:
        return False

config_string = lambda s : weechat.config_string(weechat.config_get(s))
def get_nick(s):
    """Strip nickmodes and prefix, suffix."""
    if not s: return ''
    # prefix and suffix
    prefix = config_string('irc.look.nick_prefix')
    suffix = config_string('irc.look.nick_suffix')
    if s[0] == prefix:
        s = s[1:]
    if s[-1] == suffix:
        s = s[:-1]
    # nick mode
    modes = '~+@!%'
    s = s.lstrip(modes)
    return s

def notify_msg(workaround, buffer, time, tags, display, hilight, prefix, msg):
    if workaround and 'notify_message' not in tags and 'notify_private' not in tags:
        # weechat 0.3.0 bug
        return WEECHAT_RC_OK
    #debug('  '.join((buffer, time, tags, display, hilight, prefix, 'msg_len:%s' %len(msg))),
    #        prefix='MESSAGE')
    private = 'notify_private' in tags
    if (int(hilight) or private) and int(display):
        if 'irc_action' in tags:
            prefix, _, msg = msg.partition(' ')
            msg = '%s %s' %(config_string('weechat.look.prefix_action'), msg)
        prefix = get_nick(prefix)
        if prefix not in ignore_nick \
                and msg not in ignore_text \
                and not is_displayed(buffer):
            #debug('%sSending notification: %s' %(weechat.color('lightgreen'), channel), prefix='NOTIFY')
            if not private:
                channel = weechat.buffer_get_string(buffer, 'localvar_channel')
                if channel not in ignore_channel:
                    send_notify(msg, channel=channel, nick=prefix)
            else:
                send_notify(msg, channel=prefix)
    return WEECHAT_RC_OK

def cmd_notify(data, buffer, args):
    if args:
        args = args.split()
        cmd = args[0]
        if cmd in ('test', 'quit', 'restart', 'notify'):
            if cmd == 'test':
                server.send_rpc(' '.join(args[1:]) or 'This is a test.', '#test')
            elif cmd == 'notify':
                send_notify(' '.join(args[1:]) or 'This is a test.', '#test')
            elif cmd == 'quit':
                say('Shutting down notification daemon...')
                server.quit()
            elif cmd == 'restart':
                say('Restarting notification daemon...')
                server.restart()
            return WEECHAT_RC_OK

    weechat.command('', '/help %s' %SCRIPT_COMMAND)
    return WEECHAT_RC_OK

def ignore_update(*args):
    ignore_channel._get_ignores()
    ignore_nick._get_ignores()
    ignore_text._get_ignores()
    return WEECHAT_RC_OK

def server_update(*args):
    server._create_server()
    return WEECHAT_RC_OK


if __name__ == '__main__' and import_ok and \
        weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC,
        '', ''):

    # pretty nick
    color_delimiter = weechat.color('chat_delimiters')
    color_nick = weechat.color('chat_nick')
    color_reset = weechat.color('reset')
    script_nick = '%s[%s%s%s]%s' %(color_delimiter, color_nick, SCRIPT_NAME, color_delimiter, color_reset)

    # check if we need to workaround a bug in 0.3.0
    workaround = ''
    version = weechat.info_get('version', '')
    if version == '0.3.0':
        workaround = '1'
        #debug('workaround enabled')

    for opt, val in settings.iteritems():
        if not weechat.config_is_set_plugin(opt):
            weechat.config_set_plugin(opt, val)

    ignore_channel = Ignores('ignore_channel')
    ignore_nick = Ignores('ignore_nick')
    ignore_text = Ignores('ignore_text')

    server = Server()

    weechat.hook_command(SCRIPT_COMMAND, SCRIPT_DESC, '[test [text] | notify [text] | restart | quit ]', 
"""\
   test: sends a test notification, with 'text' if provided ('text'
         is sent raw).
 notify: same as test, but the notification is sent through the
         notification queue and after formatting.
restart: forces remote daemon to restart.
   quit: forces remote daemon to shutdown, after this notifications
         won't be available and the daemon should be started again
         manually.

Setting notification ignores:
  It's possible to filter notification by channel, by nick or by
  message content, with the config options ignore_channel, 
  ignore_nick and ignore_text in plugins.var.python.%(script)s
  Each config option accepts a comma separated list of patterns.
  Wildcards '*', '?' and char groups [..] can be used.
  An ignore exception can be added by prefixing '!' in the pattern.

Examples:
  Setting 'ignore_nick' to 'troll,b[0o]t':
   will ignore notifications from troll, bot and b0t.
  Setting 'ignore_channel' to '*ubuntu*,!#ubuntu-offtopic':
   will ignore notifications from any channel with the word 'ubuntu'
   except from #ubuntu-offtopic.

Daemon:
  %(script)s script needs to connect to an external daemon for send
  notifications, which can be used in localhost or remotely.
  Download the daemon from:
  %(daemon_url)s
  and check its help with ./%(daemon)s --help.
  See also help in script file.
""" %dict(script=SCRIPT_NAME, daemon_url=DAEMON_URL, daemon=DAEMON)
            ,'test|notify|restart|quit', 'cmd_notify', '')

    weechat.hook_config('plugins.var.python.%s.ignore_*' %SCRIPT_NAME, 'ignore_update', '')
    weechat.hook_config('plugins.var.python.%s.server_*' %SCRIPT_NAME, 'server_update', '')

    weechat.hook_print('', 'notify_message', '', 1, 'notify_msg', workaround)
    weechat.hook_print('', 'notify_private', '', 1, 'notify_msg', workaround)


# vim:set shiftwidth=4 tabstop=4 softtabstop=4 expandtab textwidth=100:



# -*- coding: utf-8 -*-
#
# Copyright (C) 2017 Marcin Kurczewski <rr-@sakuya.pl>
# Copyright (C) 2017 Ricardo Ferreira <ricardo.sff@goatse.cx>
# Copyright (C) 2014 Charles Franklin <jakhead@gmail.com>
# Copyright (C) 2012 Markus Näsman <markus@botten.org>
# Copyright (C) 2011 David Flatz <david@upcs.at>
# Copyright (C) 2009 Bjorn Edstrom <be@bjrn.se>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

#
# NOTE: Blowfish and DH1080 implementation is licenced under a different
# license:
#
# Copyright (c) 2009, Bjorn Edstrom <be@bjrn.se>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#

#
# Suggestions, Bugs, ...?
# https://github.com/freshprince/weechat-fish

#
# NOTE ABOUT DH1080:
# =================
#
# Diffie-Hellman key exchange assumes that you already have
# authenticated channels between Alice and Bob.  Which means that Alice
# has to be sure that she is really talking to Bob and not to any man in
# the middle.  But since the whole idea of FiSH is that you want to
# encrypt your communication on the IRC server whose operators you do
# not trust, there is no reliable way for Alice to tell if she really is
# talking to Bob.  It could also be some rogue IRC admin impersonating
# Bob with a fake hostname and ident or even doing a MITM attack on
# DH1080.  This means you can consider using DH1080 key exchange over
# IRC utterly broken in terms of security.
#

SCRIPT_NAME = "fish"
SCRIPT_AUTHOR = "David Flatz <david@upcs.at>"
SCRIPT_VERSION = "0.9.2"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC = "FiSH for weechat"
CONFIG_FILE_NAME = SCRIPT_NAME

import_ok = True

import re
import struct
import hashlib
from os import urandom

try:
    import weechat
except ImportError:
    print "This script must be run under WeeChat."
    print "Get WeeChat now at: http://www.weechat.org/"
    import_ok = False

try:
    import Crypto.Cipher.Blowfish
except:
    print "Python Cryptography Toolkit must be installed to use fish"
    import_ok = False


#
# GLOBALS
#

fish_config_file = None
fish_config_section = {}
fish_config_option = {}
fish_keys = {}
fish_cyphers = {}
fish_DH1080ctx = {}
fish_encryption_announced = {}

fish_secure_key = ""
fish_secure_cipher = None

#
# CONFIG
#

def fish_config_reload_cb(data, config_file):
    return weechat.config_reload(config_file)


def fish_config_keys_read_cb(data, config_file, section_name, option_name,
        value):
    global fish_keys

    option = weechat.config_new_option(config_file, section_name, option_name,
            "string", "key", "", 0, 0, "", value, 0, "", "", "", "", "", "")
    if not option:
        return weechat.WEECHAT_CONFIG_OPTION_SET_ERROR

    fish_keys[option_name] = value

    return weechat.WEECHAT_CONFIG_OPTION_SET_OK_CHANGED


def fish_config_keys_write_cb(data, config_file, section_name):
    global fish_keys, fish_secure_cipher

    weechat.config_write_line(config_file, section_name, "")
    for target, key in sorted(fish_keys.iteritems()):

        if fish_secure_cipher != None:
            ### ENCRYPT Targets/Keys ###
            weechat.config_write_line(config_file,
                                      blowcrypt_pack(target, fish_secure_cipher),
                                      blowcrypt_pack(key, fish_secure_cipher))

        else:
            weechat.config_write_line(config_file, target, key)

    return weechat.WEECHAT_RC_OK


def fish_config_init():
    global fish_config_file, fish_config_section, fish_config_option
    global fish_secure_cipher

    fish_config_file = weechat.config_new(CONFIG_FILE_NAME,
            "fish_config_reload_cb", "")
    if not fish_config_file:
        return

    # look
    fish_config_section["look"] = weechat.config_new_section(fish_config_file,
        "look", 0, 0, "", "", "", "", "", "", "", "", "", "")
    if not fish_config_section["look"]:
        weechat.config_free(fish_config_file)
        return

    fish_config_option["announce"] = weechat.config_new_option(
        fish_config_file, fish_config_section["look"], "announce",
        "boolean", "annouce if messages are being encrypted or not", "", 0,
        0, "on", "on", 0, "", "", "", "", "", "")

    fish_config_option["marker"] = weechat.config_new_option(
        fish_config_file, fish_config_section["look"], "marker",
        "string", "marker for important FiSH messages", "", 0, 0,
        "O<", "O<", 0, "", "", "", "", "", "")

    fish_config_option["mark_position"] = weechat.config_new_option(
        fish_config_file, fish_config_section["look"], "mark_position",
        "integer", "put marker for encrypted messages at start or end",
        "off|begin|end",
        0,2, "off", "off", 0, "", "", "", "", "", "")

    fish_config_option["mark_encrypted"] = weechat.config_new_option(
        fish_config_file, fish_config_section["look"], "mark_encrypted",
        "string", "marker for encrypted messages", "", 0, 0,
        "*", "*", 0, "", "", "", "", "", "")

    # color
    fish_config_section["color"] = weechat.config_new_section(fish_config_file,
            "color", 0, 0, "", "", "", "", "", "", "", "", "", "")
    if not fish_config_section["color"]:
        weechat.config_free(fish_config_file)
        return

    fish_config_option["alert"] = weechat.config_new_option(
        fish_config_file, fish_config_section["color"], "alert",
        "color", "color for important FiSH message markers", "", 0, 0,
        "lightblue", "lightblue", 0, "", "", "", "", "", "")

    # secure
    fish_config_section["secure"] = weechat.config_new_section(fish_config_file,
            "secure", 0, 0, "", "", "", "", "", "", "", "", "", "")
    if not fish_config_section["secure"]:
        weechat.config_free(fish_config_file)
        return

    fish_config_option["key"] = weechat.config_new_option(
        fish_config_file, fish_config_section["secure"], "key",
        "string", "key for securing blowfish keys", "", 0, 0, "", "",
        0, "", "", "", "", "", "")

    # keys
    fish_config_section["keys"] = weechat.config_new_section(fish_config_file,
        "keys", 0, 0,
        "fish_config_keys_read_cb", "",
        "fish_config_keys_write_cb", "", "",
        "", "", "", "", "")
    if not fish_config_section["keys"]:
        weechat.config_free(fish_config_file)
        return


def fish_config_read():
    global fish_config_file

    return weechat.config_read(fish_config_file)


def fish_config_write():
    global fish_config_file

    return weechat.config_write(fish_config_file)


##
## Blowfish and DH1080 Code:
##
#
# BLOWFISH
#

class Blowfish:

    def __init__(self, key=None):
        if key:
            if len(key) > 72:
                key = key[:72]
            self.blowfish = Crypto.Cipher.Blowfish.new(key)

    def decrypt(self, data):
        return self.blowfish.decrypt(data)

    def encrypt(self, data):
        return self.blowfish.encrypt(data)


# XXX: Unstable.
def blowcrypt_b64encode(s):
    """A non-standard base64-encode."""
    B64 = "./0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"
    res = ''
    while s:
        left, right = struct.unpack('>LL', s[:8])
        for i in xrange(6):
            res += B64[right & 0x3f]
            right >>= 6
        for i in xrange(6):
            res += B64[left & 0x3f]
            left >>= 6
        s = s[8:]
    return res


def blowcrypt_b64decode(s):
    """A non-standard base64-decode."""
    B64 = "./0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"
    res = ''
    while s:
        left, right = 0, 0
        for i, p in enumerate(s[0:6]):
            right |= B64.index(p) << (i * 6)
        for i, p in enumerate(s[6:12]):
            left |= B64.index(p) << (i * 6)
        for i in range(0,4):
            res +=chr(((left & (0xFF << ((3 - i) * 8))) >> ((3 - i) * 8)))
        for i in range(0,4):
            res +=chr(((right & (0xFF << ((3 - i) * 8))) >> ((3 - i) * 8)))
        s = s[12:]
    return res


def padto(msg, length):
    """Pads 'msg' with zeroes until it's length is divisible by 'length'.
    If the length of msg is already a multiple of 'length', does nothing."""
    L = len(msg)
    if L % length:
        msg += '\x00' * (length - L % length)
    assert len(msg) % length == 0
    return msg


def blowcrypt_pack(msg, cipher):
    """."""
    return '+OK ' + blowcrypt_b64encode(cipher.encrypt(padto(msg, 8)))


def blowcrypt_unpack(msg, cipher):
    """."""
    if not (msg.startswith('+OK ') or msg.startswith('mcps ')):
        raise ValueError
    _, rest = msg.split(' ', 1)
    if len(rest) < 12:
        raise MalformedError

    if not (len(rest) %12) == 0:
        rest = rest[:-(len(rest) % 12)]

    try:
        raw = blowcrypt_b64decode(padto(rest, 12))
    except TypeError:
        raise MalformedError
    if not raw:
        raise MalformedError

    try:
        plain = cipher.decrypt(raw)
    except ValueError:
        raise MalformedError

    return plain.strip('\x00').replace('\n','')


#
# DH1080
#

g_dh1080 = 2
p_dh1080 = int('FBE1022E23D213E8ACFA9AE8B9DFAD'
               'A3EA6B7AC7A7B7E95AB5EB2DF85892'
               '1FEADE95E6AC7BE7DE6ADBAB8A783E'
               '7AF7A7FA6A2B7BEB1E72EAE2B72F9F'
               'A2BFB2A2EFBEFAC868BADB3E828FA8'
               'BADFADA3E4CC1BE7E8AFE85E9698A7'
               '83EB68FA07A77AB6AD7BEB618ACF9C'
               'A2897EB28A6189EFA07AB99A8A7FA9'
               'AE299EFA7BA66DEAFEFBEFBF0B7D8B', 16)
q_dh1080 = (p_dh1080 - 1) / 2


def dh1080_b64encode(s):
    """A non-standard base64-encode."""
    b64 = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/"
    d = [0] * len(s) * 2

    L = len(s) * 8
    m = 0x80
    i, j, k, t = 0, 0, 0, 0
    while i < L:
        if ord(s[i >> 3]) & m:
            t |= 1
        j += 1
        m >>= 1
        if not m:
            m = 0x80
        if not j % 6:
            d[k] = b64[t]
            t &= 0
            k += 1
        t <<= 1
        t %= 0x100
        #
        i += 1
    m = 5 - j % 6
    t <<= m
    t %= 0x100
    if m:
        d[k] = b64[t]
        k += 1
    d[k] = 0
    res = ''
    for q in d:
        if q == 0:
            break
        res += q
    return res


def dh1080_b64decode(s):
    """A non-standard base64-encode."""
    b64 = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/"
    buf = [0] * 256
    for i in range(64):
        buf[ord(b64[i])] = i

    L = len(s)
    if L < 2:
        raise ValueError
    for i in reversed(range(L - 1)):
        if buf[ord(s[i])] == 0:
            L -= 1
        else:
            break
    if L < 2:
        raise ValueError

    d = [0] * L
    i, k = 0, 0
    while True:
        i += 1
        if k + 1 < L:
            d[i - 1] = buf[ord(s[k])] << 2
            d[i - 1] %= 0x100
        else:
            break
        k += 1
        if k < L:
            d[i - 1] |= buf[ord(s[k])] >> 4
        else:
            break
        i += 1
        if k + 1 < L:
            d[i - 1] = buf[ord(s[k])] << 4
            d[i - 1] %= 0x100
        else:
            break
        k += 1
        if k < L:
            d[i - 1] |= buf[ord(s[k])] >> 2
        else:
            break
        i += 1
        if k + 1 < L:
            d[i - 1] = buf[ord(s[k])] << 6
            d[i - 1] %= 0x100
        else:
            break
        k += 1
        if k < L:
            d[i - 1] |= buf[ord(s[k])] % 0x100
        else:
            break
        k += 1
    return ''.join(map(chr, d[0:i - 1]))


def dh_validate_public(public, q, p):
    """See RFC 2631 section 2.1.5."""
    return 1 == pow(public, q, p)


class DH1080Ctx:
    """DH1080 context."""
    def __init__(self):
        self.public = 0
        self.private = 0
        self.secret = 0
        self.state = 0

        bits = 1080
        while True:
            self.private = bytes2int(urandom(bits / 8))
            self.public = pow(g_dh1080, self.private, p_dh1080)
            if 2 <= self.public <= p_dh1080 - 1 and \
               dh_validate_public(self.public, q_dh1080, p_dh1080) == 1:
                break


def dh1080_pack(ctx):
    """."""
    cmd = None
    if ctx.state == 0:
        ctx.state = 1
        cmd = "DH1080_INIT "
    else:
        cmd = "DH1080_FINISH "
    return cmd + dh1080_b64encode(int2bytes(ctx.public))


def dh1080_unpack(msg, ctx):
    """."""
    if not msg.startswith("DH1080_"):
        raise ValueError

    invalidmsg = "Key does not validate per RFC 2631. This check is not " \
                 "performed by any DH1080 implementation, so we use the key " \
                 "anyway. See RFC 2785 for more details."

    if ctx.state == 0:
        if not msg.startswith("DH1080_INIT "):
            raise MalformedError
        ctx.state = 1
        try:
            cmd, public_raw = msg.split(' ', 1)
            public = bytes2int(dh1080_b64decode(public_raw))

            if not 1 < public < p_dh1080:
                raise MalformedError

            if not dh_validate_public(public, q_dh1080, p_dh1080):
                #print invalidmsg
                pass

            ctx.secret = pow(public, ctx.private, p_dh1080)
        except:
            raise MalformedError

    elif ctx.state == 1:
        if not msg.startswith("DH1080_FINISH "):
            raise MalformedError
        ctx.state = 1
        try:
            cmd, public_raw = msg.split(' ', 1)
            public = bytes2int(dh1080_b64decode(public_raw))

            if not 1 < public < p_dh1080:
                raise MalformedError

            if not dh_validate_public(public, q_dh1080, p_dh1080):
                #print invalidmsg
                pass

            ctx.secret = pow(public, ctx.private, p_dh1080)
        except:
            raise MalformedError

    return True


def dh1080_secret(ctx):
    """."""
    if ctx.secret == 0:
        raise ValueError
    return dh1080_b64encode(sha256(int2bytes(ctx.secret)))


def bytes2int(b):
    """Variable length big endian to integer."""
    n = 0
    for p in b:
        n *= 256
        n += ord(p)
    return n


def int2bytes(n):
    """Integer to variable length big endian."""
    if n == 0:
        return '\x00'
    b = ''
    while n:
        b = chr(n % 256) + b
        n /= 256
    return b


def sha256(s):
    """sha256"""
    return hashlib.sha256(s).digest()


##
##  END Blowfish and DH1080 Code
##
#
# HOOKS
#

def fish_secure_key_cb(data, option, value):
    global fish_secure_key, fish_secure_cipher

    fish_secure_key = weechat.config_string(
        weechat.config_get("fish.secure.key")
    )

    if fish_secure_key == "":
        fish_secure_cipher = None
        return weechat.WEECHAT_RC_OK

    if fish_secure_key[:6] == "${sec.":
        decrypted = weechat.string_eval_expression(
            fish_secure_key, {}, {}, {}
        )
        if decrypted:
            fish_secure_cipher = Blowfish(decrypted)
            return weechat.WEECHAT_RC_OK
        else:
            weechat.config_option_set(fish_config_option["key"], "", 0)
            weechat.prnt("", "Decrypt sec.conf first\n")
            return weechat.WEECHAT_RC_OK

    if fish_secure_key != "":
        fish_secure_cipher = Blowfish(fish_secure_key)

    return weechat.WEECHAT_RC_OK


def fish_modifier_in_notice_cb(data, modifier, server_name, string):
    global fish_DH1080ctx, fish_keys, fish_cyphers

    match = re.match(
        r"^(:(.*?)!.*? NOTICE (.*?) :)((DH1080_INIT |DH1080_FINISH |\+OK |mcps )?.*)$",
        string)
    #match.group(0): message
    #match.group(1): msg without payload
    #match.group(2): source
    #match.group(3): target
    #match.group(4): msg
    #match.group(5): DH1080_INIT |DH1080_FINISH
    if not match or not match.group(5):
        return string

    if match.group(3) != weechat.info_get("irc_nick", server_name):
        return string

    target = "%s/%s" % (server_name, match.group(2))
    targetl = ("%s/%s" % (server_name, match.group(2))).lower()
    buffer = weechat.info_get("irc_buffer", "%s,%s" % (
            server_name, match.group(2)))

    if match.group(5) == "DH1080_FINISH " and targetl in fish_DH1080ctx:
        if not dh1080_unpack(match.group(4), fish_DH1080ctx[targetl]):
            fish_announce_unencrypted(buffer, target)
            return string

        fish_alert(buffer, "Key exchange for %s sucessful" % target)

        fish_keys[targetl] = dh1080_secret(fish_DH1080ctx[targetl])
        if targetl in fish_cyphers:
            del fish_cyphers[targetl]
        del fish_DH1080ctx[targetl]

        return ""

    if match.group(5) == "DH1080_INIT ":
        fish_DH1080ctx[targetl] = DH1080Ctx()

        msg = ' '.join(match.group(4).split()[0:2])

        if not dh1080_unpack(msg, fish_DH1080ctx[targetl]):
            fish_announce_unencrypted(buffer, target)
            return string

        reply = dh1080_pack(fish_DH1080ctx[targetl])

        fish_alert(buffer, "Key exchange initiated by %s. Key set." % target)

        weechat.command(buffer, "/mute -all notice %s %s" % (
                match.group(2), reply))

        fish_keys[targetl] = dh1080_secret(fish_DH1080ctx[targetl])
        if targetl in fish_cyphers:
            del fish_cyphers[targetl]
        del fish_DH1080ctx[targetl]

        return ""

    if match.group(5) in ["+OK ", "mcps "]:
        if targetl not in fish_keys:
            fish_announce_unencrypted(buffer, target)
            return string

        if targetl not in fish_cyphers:
            b = Blowfish(fish_keys[targetl])
            fish_cyphers[targetl] = b
        else:
            b = fish_cyphers[targetl]

        clean = blowcrypt_unpack(match.group(4), b)

        fish_announce_encrypted(buffer, target)

        return "%s%s" % (match.group(1), fish_msg_w_marker(clean))

    fish_announce_unencrypted(buffer, target)

    return string


def fish_modifier_in_privmsg_cb(data, modifier, server_name, string):
    global fish_keys, fish_cyphers

    match = re.match(
        r"^(:(.*?)!.*? PRIVMSG (.*?) :)(\x01ACTION )?((\+OK |mcps )?.*?)(\x01)?$",
        string)
    #match.group(0): message
    #match.group(1): msg without payload
    #match.group(2): source
    #match.group(3): target
    #match.group(4): action
    #match.group(5): msg
    #match.group(6): +OK |mcps
    if not match:
        return string

    if match.group(3) == weechat.info_get("irc_nick", server_name):
        dest = match.group(2)
    else:
        dest = match.group(3)
    target = "%s/%s" % (server_name, dest)
    targetl = ("%s/%s" % (server_name, dest)).lower()
    buffer = weechat.info_get("irc_buffer", "%s,%s" % (server_name, dest))

    if not match.group(6):
        fish_announce_unencrypted(buffer, target)

        return string

    if targetl not in fish_keys:
        fish_announce_unencrypted(buffer, target)

        return string

    fish_announce_encrypted(buffer, target)

    if targetl not in fish_cyphers:
        b = Blowfish(fish_keys[targetl])
        fish_cyphers[targetl] = b
    else:
        b = fish_cyphers[targetl]
    clean = blowcrypt_unpack(match.group(5), b)

    if not match.group(4):
        return "%s%s" % (match.group(1), fish_msg_w_marker(clean))

    return "%s%s%s\x01" % (match.group(1), match.group(4), fish_msg_w_marker(clean))


def fish_modifier_in_topic_cb(data, modifier, server_name, string):
    global fish_keys, fish_cyphers

    match = re.match(r"^(:.*?!.*? TOPIC (.*?) :)((\+OK |mcps )?.*)$", string)
    #match.group(0): message
    #match.group(1): msg without payload
    #match.group(2): channel
    #match.group(3): topic
    #match.group(4): +OK |mcps
    if not match:
        return string

    target = "%s/%s" % (server_name, match.group(2))
    targetl = ("%s/%s" % (server_name, match.group(2))).lower()
    buffer = weechat.info_get("irc_buffer", "%s,%s" % (server_name,
            match.group(2)))

    if targetl not in fish_keys or not match.group(4):
        fish_announce_unencrypted(buffer, target)

        return string

    if targetl not in fish_cyphers:
        b = Blowfish(fish_keys[targetl])
        fish_cyphers[targetl] = b
    else:
        b = fish_cyphers[targetl]
    clean = blowcrypt_unpack(match.group(3), b)

    fish_announce_encrypted(buffer, target)

    return "%s%s" % (match.group(1), fish_msg_w_marker(clean))


def fish_modifier_in_332_cb(data, modifier, server_name, string):
    global fish_keys, fish_cyphers

    match = re.match(r"^(:.*? 332 .*? (.*?) :)((\+OK |mcps )?.*)$", string)
    if not match:
        return string

    target = "%s/%s" % (server_name, match.group(2))
    targetl = ("%s/%s" % (server_name, match.group(2))).lower()
    buffer = weechat.info_get("irc_buffer", "%s,%s" % (server_name,
            match.group(2)))

    if targetl not in fish_keys or not match.group(4):
        fish_announce_unencrypted(buffer, target)

        return string

    if targetl not in fish_cyphers:
        b = Blowfish(fish_keys[targetl])
        fish_cyphers[targetl] = b
    else:
        b = fish_cyphers[targetl]

    clean = blowcrypt_unpack(match.group(3), b)

    fish_announce_encrypted(buffer, target)

    return "%s%s" % (match.group(1), fish_msg_w_marker(clean))


def fish_modifier_out_privmsg_cb(data, modifier, server_name, string):
    global fish_keys, fish_cyphers

    match = re.match(r"^(PRIVMSG (.*?) :)(.*)$", string)
    if not match:
        return string

    target = "%s/%s" % (server_name, match.group(2))
    targetl = ("%s/%s" % (server_name, match.group(2))).lower()
    buffer = weechat.info_get("irc_buffer", "%s,%s" % (server_name,
            match.group(2)))

    if targetl not in fish_keys:
        fish_announce_unencrypted(buffer, target)

        return string

    if targetl not in fish_cyphers:
        b = Blowfish(fish_keys[targetl])
        fish_cyphers[targetl] = b
    else:
        b = fish_cyphers[targetl]
    cypher = blowcrypt_pack(fish_msg_wo_marker(match.group(3)), b)

    fish_announce_encrypted(buffer, target)

    return "%s%s" % (match.group(1), cypher)


def fish_modifier_out_topic_cb(data, modifier, server_name, string):
    global fish_keys, fish_cyphers

    match = re.match(r"^(TOPIC (.*?) :)(.*)$", string)
    if not match:
        return string
    if not match.group(3):
        return string

    target = "%s/%s" % (server_name, match.group(2))
    targetl = ("%s/%s" % (server_name, match.group(2))).lower()
    buffer = weechat.info_get("irc_buffer", "%s,%s" % (server_name,
            match.group(2)))

    if targetl not in fish_keys:
        fish_announce_unencrypted(buffer, target)

        return string

    if targetl not in fish_cyphers:
        b = Blowfish(fish_keys[targetl])
        fish_cyphers[targetl] = b
    else:
        b = fish_cyphers[targetl]
    cypher = blowcrypt_pack(match.group(3), b)

    fish_announce_encrypted(buffer, target)

    return "%s%s" % (match.group(1), cypher)


def fish_modifier_input_text(data, modifier, server_name, string):
    if weechat.string_is_command_char(string):
        return string
    buffer = weechat.current_buffer()
    name = weechat.buffer_get_string(buffer, "name")
    target = name.replace(".", "/")
    targetl = target.lower()
    if targetl not in fish_keys:
        return string
    return "%s" % (fish_msg_w_marker(string))


def fish_unload_cb():
    fish_config_write()

    return weechat.WEECHAT_RC_OK


#
# COMMANDS
#

def fish_cmd_blowkey(data, buffer, args):
    global fish_keys, fish_cyphers, fish_DH1080ctx
    global fish_config_option, fish_secure_cipher

    if args == "" or args == "list":
        fish_list_keys(buffer)

        return weechat.WEECHAT_RC_OK

    elif args =="genkey":
        fish_secure_genkey(buffer)
        return weechat.WEECHAT_RC_OK

    argv = args.split(" ")

    if (len(argv) > 2 and argv[1] == "-server"):
        server_name = argv[2]
        del argv[2]
        del argv[1]
        pos = args.find(" ")
        pos = args.find(" ", pos + 1)
        args = args[pos+1:]
    else:
        server_name = weechat.buffer_get_string(buffer, "localvar_server")

    buffer_type = weechat.buffer_get_string(buffer, "localvar_type")
    # if no target user has been specified grab the one from the buffer if it is private
    if argv[0] == "exchange" and len(argv) == 1 and buffer_type == "private":
        target_user = weechat.buffer_get_string(buffer, "localvar_channel")
    elif argv[0] == "set" and (buffer_type == "private" or buffer_type == "channel") and len(argv) == 2:
        target_user = weechat.buffer_get_string(buffer, "localvar_channel")
    elif len(argv) < 2:
        return weechat.WEECHAT_RC_ERROR
    else:
        target_user = argv[1]

    argv2eol = ""
    pos = args.find(" ")
    if pos:
        pos = args.find(" ", pos + 1)
        if pos > 0:
            argv2eol = args[pos + 1:]
        else:
            argv2eol = args[args.find(" ") +1:]

    target = "%s/%s" % (server_name, target_user)
    targetl = ("%s/%s" % (server_name, target_user)).lower()

    if argv[0] == "set":
        fish_keys[targetl] = argv2eol

        if targetl in fish_cyphers:
            del fish_cyphers[targetl]

        weechat.prnt(buffer, "set key for %s to %s" % (target, argv2eol))

        return weechat.WEECHAT_RC_OK

    if argv[0] == "remove":
        if not len(argv) == 2:
            return weechat.WEECHAT_RC_ERROR

        if targetl not in fish_keys:
            return weechat.WEECHAT_RC_ERROR

        del fish_keys[targetl]

        if targetl in fish_cyphers:
            del fish_cyphers[targetl]

        weechat.prnt(buffer, "removed key for %s" % target)

        return weechat.WEECHAT_RC_OK

    if argv[0] == "exchange":
        if server_name == "":
            return weechat.WEECHAT_RC_ERROR

        weechat.prnt(buffer, "Initiating DH1080 Exchange with %s" % target)
        fish_DH1080ctx[targetl] = DH1080Ctx()
        msg = dh1080_pack(fish_DH1080ctx[targetl])
        weechat.command(buffer, "/mute -all notice -server %s %s %s" % (server_name, target_user, msg))

        return weechat.WEECHAT_RC_OK



    return weechat.WEECHAT_RC_ERROR


#
# HELPERS
#


def fish_secure():
    global fish_secure_key, fish_secure_cipher

    fish_secure_key = weechat.config_string(fish_config_option["key"])

    # if blank, do nothing
    if fish_secure_key == "":
        fish_success()
        return

    # if ${sec.data.fish}, check if sec.conf is decrypted
    # and decrypt
    elif fish_secure_key[:6] == "${sec.":
        decrypted = weechat.string_eval_expression(
            fish_secure_key, {}, {}, {}
        )

        if decrypted:
            fish_secure_cipher = Blowfish(decrypted)
            fish_decrypt_keys()
            fish_success()
            return

        else:
            global SCRIPT_NAME
            fish_secure_error()
            weechat.command(weechat.current_buffer(),
                            "/wait 1ms /python unload %s" % SCRIPT_NAME)
            return

    # if key is neither ${sec.data.fish} or ""
    # encrypt/decrypt with user supplied, plain text key
    if fish_secure_key != "":
        fish_secure_cipher = Blowfish(fish_secure_key)
        fish_decrypt_keys()
        fish_success()
        return


def fish_decrypt_keys():
    global fish_keys, fish_secure_cipher
    global fish_cyphers

    fish_keys_tmp = {}
    for target, key in fish_keys.iteritems():
        ### DECRYPT Targets/Keys ###
        fish_keys_tmp[blowcrypt_unpack(
            target,
            fish_secure_cipher)] = blowcrypt_unpack(key,
                                                  fish_secure_cipher)

    fish_keys = fish_keys_tmp


def fish_success():
    weechat.prnt("",
                 "%s%sblowkey: succesfully loaded\n" % (
                     weechat.prefix("join"),
                     weechat.color("_green"))
    )


def fish_secure_error():
    """print error message if secdata not decrypted"""

    message = ("\n%s%sblowkey:%s unable to recover key from sec.conf\n"
               "%s%sblowkey:%s fish.py %sNOT LOADED\n"
               "%s%sblowkey:%s decrypt secured data first\n"
               "%s%sblowkey:%s then reload fish.py\n\n") % (
                   weechat.prefix("error"),
                   weechat.color("underline"),
                   weechat.color("reset"),
                   weechat.prefix("error"),
                   weechat.color("underline"),
                   weechat.color("reset"),
                   weechat.color("*red"),
                   weechat.prefix("error"),
                   weechat.color("underline"),
                   weechat.color("reset"),
                   weechat.prefix("error"),
                   weechat.color("underline"),
                   weechat.color("reset")
               )

    weechat.prnt("", "%s" % message)


def fish_secure_genkey(buffer):
    global fish_secure_cipher, fish_config_option

    newKey = blowcrypt_b64encode(urandom(32))

    # test to see if sec.conf decrypted
    weechat.command(buffer, "/secure set fish test")
    decrypted = weechat.string_eval_expression(
        "${sec.data.fish}", {}, {}, {}
    )

    if decrypted == "test":
        weechat.config_option_set(fish_config_option["key"],
                                  "${sec.data.fish}", 0)
        fish_secure_cipher = Blowfish(newKey)
        weechat.command(buffer, "/secure set fish %s" % newKey)


def fish_announce_encrypted(buffer, target):
    global fish_encryption_announced, fish_config_option

    if (not weechat.config_boolean(fish_config_option['announce']) or
        fish_encryption_announced.get(target)):
        return

    (server, nick) = target.split("/")

    if (weechat.info_get("irc_is_nick", nick) and
            weechat.buffer_get_string(buffer, "localvar_type") != "private"):
        # if we get a private message and there no buffer yet, create one and
        # jump back to the previous buffer
        weechat.command(buffer, "/mute -all query %s" % nick)
        buffer = weechat.info_get("irc_buffer", "%s,%s" % (server, nick))
        weechat.command(buffer, "/input jump_previously_visited_buffer")

    fish_alert(buffer, "Messages to/from %s are encrypted." % target)

    fish_encryption_announced[target] = True


def fish_announce_unencrypted(buffer, target):
    global fish_encryption_announced, fish_config_option

    if (not weechat.config_boolean(fish_config_option['announce']) or
            not fish_encryption_announced.get(target)):
        return

    fish_alert(buffer, "Messages to/from %s are %s*not*%s encrypted." % (
            target,
            weechat.color(weechat.config_color(fish_config_option["alert"])),
            weechat.color("chat")))

    del fish_encryption_announced[target]


def fish_alert(buffer, message):
    mark = "%s%s%s\t" % (
            weechat.color(weechat.config_color(fish_config_option["alert"])),
            weechat.config_string(fish_config_option["marker"]),
            weechat.color("chat"))

    weechat.prnt(buffer, "%s%s" % (mark, message))


def fish_list_keys(buffer):
    global fish_keys

    weechat.prnt(buffer, "\tFiSH Keys: form target(server): key")

    if len(fish_keys) == 0:
        weechat.prnt(buffer, "NO KEYS!\n")
        return

    for (target, key) in sorted(fish_keys.iteritems()):
        (server, nick) = target.split("/")
        weechat.prnt(buffer, "\t%s(%s): %s" % (nick, server, key))


def fish_msg_w_marker(msg):
    marker = weechat.config_string(fish_config_option["mark_encrypted"])
    if weechat.config_string(fish_config_option["mark_position"]) == "end":
        return "%s%s" % (msg, marker)
    elif weechat.config_string(fish_config_option["mark_position"]) == "begin":
        return "%s%s" % (marker, msg)
    else:
        return msg


def fish_msg_wo_marker(msg):
    marker = weechat.config_string(fish_config_option["mark_encrypted"])
    if weechat.config_string(fish_config_option["mark_position"]) == "end":
        return msg[0:-len(marker)]
    elif weechat.config_string(fish_config_option["mark_position"]) == "begin":
        return msg[len(marker):]
    else:
        return msg
#
# MAIN
#

if (__name__ == "__main__" and import_ok and
        weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION,
            SCRIPT_LICENSE, SCRIPT_DESC, "fish_unload_cb", "")):

    weechat.hook_command("blowkey", "Manage FiSH keys",
            "[list] | [genkey] |set [-server <server>] [<target>] <key> "
            "| remove [-server <server>] <target> "
            "| exchange [-server <server>] [<nick>]",
            "Add, change or remove key for target or perform DH1080\n"
            "keyexchange with <nick>.\n"
            "Target can be a channel or a nick.\n"
            "\n"
            "Without arguments this command lists all keys.\n"
            "\n"
            "Examples:\n"
            "Set the key for a channel: /blowkey set -server freenet #blowfish key\n"
            "Remove the key:            /blowkey remove #blowfish\n"
            "Set the key for a query:   /blowkey set nick secret+key\n"
            "List all keys:             /blowkey\n\n"
            "\n** stores keys in plaintext by default **\n\n"
            "DH1080:                    /blowkey exchange nick\n"
            "\nPlease read the source for a note about DH1080 key exchange\n",
            "list"
            "|| genkey"
            "|| set %(irc_channel)|%(nicks)|-server %(irc_servers) %- "
            "|| remove %(irc_channel)|%(nicks)|-server %(irc_servers) %- "
            "|| exchange %(nick)|-server %(irc_servers) %-",
            "fish_cmd_blowkey", "")

    fish_config_init()
    fish_config_read()
    fish_secure()

    weechat.hook_modifier("irc_in_notice", "fish_modifier_in_notice_cb", "")
    weechat.hook_modifier("irc_in_privmsg", "fish_modifier_in_privmsg_cb", "")
    weechat.hook_modifier("irc_in_topic", "fish_modifier_in_topic_cb", "")
    weechat.hook_modifier("irc_in_332", "fish_modifier_in_332_cb", "")
    weechat.hook_modifier("irc_out_privmsg", "fish_modifier_out_privmsg_cb", "")
    weechat.hook_modifier("irc_out_topic", "fish_modifier_out_topic_cb", "")
    weechat.hook_modifier("input_text_for_buffer", "fish_modifier_input_text", "")
    weechat.hook_config("fish.secure.key", "fish_secure_key_cb", "")



# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 Wil Clouser <clouserw@micropipes.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

# Passive aggressively auto-replies to private messages which say only the word
# 'ping' thus escalating the unwinnable war of people pinging without saying anything
# else.
#
# History:
#
#   2013-01-11, Wil Clouser <clouserw@micropipes.com>:
#       v0.1: Initial release
#   2015-10-30, Stanislav Ochotnicky <sochotnicky@gmail.com>
#       v0.2: Reply to public pings as well and add more informative pong
#

SCRIPT_NAME    = "autopong"
SCRIPT_AUTHOR  = "Wil Clouser <clouserw@micropipes.com>"
SCRIPT_VERSION = "0.2"
SCRIPT_LICENSE = "MIT"
SCRIPT_DESC    = "Auto-replies to 'ping' queries"

import_ok = True

# This can be changed with `/set plugins.var.python.autopong.reply_text`
defaults = {
  "reply_text": "pong (https://blogs.gnome.org/markmc/2014/02/20/naked-pings/)"
}

try:
   import weechat as w
except:
   print "Script must be run under weechat. http://www.weechat.org"
   import_ok = False


def msg_cb(data, buffer, date, tags, displayed, is_hilight, prefix, msg):
  reply = w.config_get_plugin('reply_text')
  if not w.buffer_get_string(buffer, "localvar_type") == "private":
    reply = prefix + ": " + reply
    if is_hilight and msg.endswith('ping'):
      w.command(buffer, reply)
  elif msg == 'ping':
      w.command(buffer, reply)

  return w.WEECHAT_RC_OK

if __name__ == "__main__" and import_ok:
  if w.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE,
                SCRIPT_DESC, "", ""):
    for k, v in defaults.iteritems():
      if not w.config_is_set_plugin(k):
        w.config_set_plugin(k, v)

    w.hook_print("", "notify_message", "ping", 1, "msg_cb", "")
    w.hook_print("", "notify_private", "ping", 1, "msg_cb", "")



"""
:Author: P Hargrave <resixianatgmaildotcom>

:What it does:
Allows control of an mpd server from within weechat.

Also provides the 'now playing' (np) command to publish
currently playing track information into the current buffer.

:Usage:
    /mpc

    * When called with no command 'np' is assumed

    /mpc <command> <arg1 arg2 ...>

    *Arguments may be comma or space separated.

:Configuration:
==============|==========================================
Variable name | Description [Default Value]
==============|==========================================

format.........This is a string that will be used to show
               the 'now playing' banner.
               Use `$` to precede any variable name.

               Several keys are already avaialable just
               from the mpd.MPDClient() `currentsong`
               command. (So set verbose=1 then do a
               `mpc currentsong` to see all the keys)

               As a convenience the following are also
               available:
                $title_or_file,
                $length_min, $length_sec, $pct,
                $pos_min, $pos_sec,
                $bitrate

host...........The host where your mpd runs ["localhost"]
password.......The password used to connect to mpd [""]
port...........The port to connect to mpd (usually 6600)
verbose........If set then lots more will be printed
ch_pause.......These are just inserted in the short_status
ch_play        template displayed whenever an action is
ch_stop        performed.
autoswitch.....If set automatically switch to created
               buffers (e.g. playlist)

To restore the default value of any config variable just
/unset <the_variable> and reload the plugin.

:Requires:
 python-mpd: Python MPD client library
             http://mpd.wikia.com/wiki/ClientLib:python-mpd

:Etc...:
This script was originally devised while hacking mpdnp.py
script by Henning Hasemann. Since there is basically nothing
left of the original and all the controls are new I decided
'mpc' was more appropriate now.

All the commands are those available from the mpd.py MPDClient
object, mostly I have done little to wrap these for weechat,
but a few common ones can show some basic information in the
current buffer (e.g. when you pause/play/stop). Unfortunately
these functions are not documented afaik and so I can't even
be sure of the arguments or types, so the user may need to
do some experimenting ;)

Released under GPL license.

2013-01-03, Sebastien Helleu <flashcode@flashtux.org>:
    version 0.3: fix buffer used to print message (use buffer in command
                 callback, not current buffer)
"""

import weechat as wc
from mpd import MPDClient, CommandError
from string import Template
from os.path import basename, splitext
from operator import itemgetter

DEFAULT_FMT = \
    "/me is listening to: $artist - $title_or_file ($length_min:$length_sec)"

wc.register("mpc", "Perry Hargrave", "0.3", "GPL", "mpc for weechat", "", "")

DEFAULT_CONFIG = {
    "host"      : "localhost",
    "port"      : "6600",
    "format"    : DEFAULT_FMT,
    "password"  : "",
    "verbose"   : "1",
    "debug"     : "0",
    "ch_pause"  : "||",
    "ch_play"   : ">>",
    "ch_stop"   : "--",
    "shortstats": "MPC : $state : $artist : $title",
    "playinfo"  : "$pos: $artist : $title : $album : $track : $time",
    "autoswitch": "0",
}

class __MPC(object):
    _mpdc   = None
    _host   = DEFAULT_CONFIG['host']
    _port   = DEFAULT_CONFIG['port']
    verbose = bool(int(DEFAULT_CONFIG['verbose']))
    debug   = bool(int(DEFAULT_CONFIG['debug']))

    # Switch automatically to created buffers
    autoswitch = bool(int(DEFAULT_CONFIG['autoswitch']))

    # State symbols
    state_chars = {
            'pause':DEFAULT_CONFIG['ch_pause'],
            'play':DEFAULT_CONFIG['ch_play'],
            'stop':DEFAULT_CONFIG['ch_stop'],
            }

    def __init__(self, wc_buffer=None):
        object.__init__(self)
        self._mpdc = MPDClient()
        self._commands = self._mpdc._commands
        self.wcb = wc_buffer

        self.verbose = bool(int((wc.config_get_plugin("verbose"))))
        self.debug = bool(int((wc.config_get_plugin("debug"))))
        self.autoswitch = bool(int((wc.config_get_plugin("autoswitch"))))

        for k, v in self.state_chars.iteritems():
            self.state_chars[k] = wc.config_get_plugin('ch_' + k)

        self._commands['np'] = None

    @property
    def shortstats(self): return wc.config_get_plugin("shortstats")

    def __getattribute__(self, attr):
        try:
            return object.__getattribute__(self, attr)
        except AttributeError:
            return self._mpdc.__getattr__(attr)

    def _get_status(self, key):
        return self._mpdc.status()[key]

    def _print_current(f):
        """Do `f`, then prints the short_status in the buffer.
        """
        def pf(self, *args, **kwargs):
            robj = f(self, *args, **kwargs)
            if not self.verbose: return robj

            # Show the status in the current buffer
            csong = self._mpdc.currentsong()
            csong.update({'state':self.state_chars[self._get_status('state')]})
            wc.prnt(self.wcb or wc.current_buffer(),
                    Template(self.shortstats).safe_substitute(csong))
            return robj

        return pf

    def connect(self):
        self._host = wc.config_get_plugin("host")
        self._port = int(wc.config_get_plugin("port"))
        self._mpdc.connect(host=self._host, port=self._port)
        pw = wc.config_get_plugin("password")
        if len(pw) > 0: self._mpdc.password(pw)

        if self.debug: wc.prnt(self.wcb or wc.current_buffer(),
                               'mpc debug: Connected')

    def currentsong(self):
        ds = self._mpdc.currentsong()
        itime = int(ds['time'])
        ipos  = int(ds['pos'])
        pct   = int(100 * (ipos / itime))

        ds.update({
            "title_or_file" : ds['title'] or splitext(basename(ds['file']))[0],
            "pos_sec"       : "%02d" % (ipos / 60),
            "pos_min"       : str(ipos / 60),
            "length_sec"    : "%02d" % (itime % 60),
            "length_min"    : str(itime / 60),
            "pct"           : "%2.0f" % pct,
            "bitrate"       : self._get_status('bitrate') + "kbps",
            })

        return ds

    def np(self):
        """Pushes result of np template substitution to current buffer.
        """
        ds  = self.currentsong()
        if len(ds) == 0:
            wc.prnt(self.wcb or wc.current_buffer(), "MPC: ERROR: mpd is stopped")
            return
        wc.command(self.wcb or wc.current_buffer(),
                   Template(wc.config_get_plugin("format")).safe_substitute(ds))

    @_print_current
    def next(self):
        self._mpdc.next()

    @_print_current
    def pause(self):
        self._mpdc.pause()

    @_print_current
    def play(self, *args):
        self._mpdc.play()

    def playlist(self, *args):
        def ifn( b, s, d): wc.prnt(b, Template(s).safe_substitute(d))
        def cfn(): wc.prnt(None, "mpc closing playlist buffer")
        new_buf = wc.buffer_new('mpc: playlist', "ifn", "", "cfn", "")
        wc.buffer_set(new_buf, "localvar_set_no_log", "1")

        pl = self._mpdc.playlist()
        for line in pl:
            wc.prnt(new_buf, line)

        wc.buffer_set(new_buf, "display", "1")
        return pl

    def playlistinfo(self, sortkey='pos'):
        """Shows playlist information sorted by key
        """
        new_buf = wc.buffer_search("", "mpc: playlist")
        if len(new_buf) == 0:
            new_buf = wc.buffer_new('mpc: playlist', "", "", "", "")

        pl = self._mpdc.playlistinfo()
        try:
            # Numerical sort
            spl = sorted(pl,
                         cmp=lambda x,y: cmp(int(x), int(y)),
                         key=itemgetter(sortkey))
        except ValueError:
            # Alpha sort
            lcmp = lambda x,y: cmp(x.lower(), y.lower())
            spl = sorted(pl,
                         cmp=lambda x,y: cmp(x.lower(), y.lower()),
                         key=itemgetter(sortkey))

        t = Template(wc.config_get_plugin("playinfo"))
        for line in spl:
            wc.prnt(new_buf, t.safe_substitute(line))

        return pl

    @_print_current
    def previous(self):
        self._mpdc.previous()

    def random(self, *args):
        """Toggles randomness if no argument is given.
        """
        if len(args) == 0:
            args = [int(not int(self._get_status('random'))),]

        self._mpdc.random(*args)

    @_print_current
    def stop(self, *args):
        self._mpdc.stop()


def set_config():
    for k, v in DEFAULT_CONFIG.iteritems():
        if not wc.config_get_plugin(k):
            wc.config_set_plugin(k, v)


def control(data, wc_buf, cmd):
    """MPC-like controls from weechat.
    """
    debug = bool(int((wc.config_get_plugin("debug"))))

    if len(cmd) < 1: cmd = 'np'
    mpc = __MPC(wc_buffer=wc_buf)
    mpc.connect()
    cmdlist = cmd.split(' ')
    if len(cmdlist) < 2:
        cmdlist = cmd.split(',')

    try:
        f = mpc.__getattribute__(cmdlist[0])
    except AttributeError:
        wc.prnt(wc_buf, 'MPC: ERROR: Invalid command: %s' %cmdlist[0])
        return wc.WEECHAT_RC_ERROR

    if mpc.debug: wc.prnt(wc_buf, 'mpc debug: f=%s' %f)
    if mpc.debug: wc.prnt(wc_buf, 'mpc debug: cmdlist=%s' %cmdlist[1:])

    # FIXME: catch errors here?
    robj = f(*cmdlist[1:])

    if debug: wc.prnt(wc_buf, 'mpc debug: got: %s' %robj)

    return wc.WEECHAT_RC_OK


set_config()
mpc = __MPC()
CMD_LIST = mpc._commands.keys()
del mpc


wc.hook_command("mpc",
                __doc__,
                control.__doc__,
                '| '.join(CMD_LIST[0:]),
                '|| '.join(CMD_LIST[0:]),
                "control", "")
